    
PREAMBLE = """\
<?xml version="1.0" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" 
"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">

<svg 
    width="%(width)fmm" 
    height="%(height)fmm" 
    viewbox="0 0 %(width)fmm %(height)fmm" 
    version="1.1"
    xmlns="http://www.w3.org/2000/svg">
<g transform="scale(%(scale)f,%(scale)f) translate(%(trans_x)f,%(trans_y)f)">
<rect x="%(neg_trans_x)f" y="%(neg_trans_y)f" width="%(width)f" height="%(height)f" style="fill:#ffffff"/>
"""

POSTAMBLE = """\
</g></svg>
"""

class SVG:
    def __init__(self):
        self.min_x = None
        self.max_x = None
        self.min_y = None
        self.max_y = None
        self.commands = [ ]
    
    def save(self, filename):
        # Assume 90dpi (inkscape default
        scale = 90 / 25.4
        pad = max(self.max_x-self.min_x,self.max_y-self.min_y) * 0.1
        width = (self.max_x-self.min_x+pad*2)
        height = (self.max_y-self.min_y+pad*2)
        trans_x = -self.min_x+pad
        trans_y = -self.min_y+pad        
        neg_trans_x = -trans_x
        neg_trans_y = -trans_y
        
        with open(filename, 'w') as f:
            f.write(PREAMBLE % locals())
            for item in self.commands:
                f.write(item + '\n')
            f.write(POSTAMBLE)
    
    def require(self, x,y):
        if self.min_x is None:
            self.min_x = self.max_x = x
            self.min_y = self.max_y = y
        else:
            self.min_x = min(self.min_x,x)
            self.max_x = max(self.max_x,x)
            self.min_y = min(self.min_y,y)
            self.max_y = max(self.max_y,y)
   
    def circle(self, x,y,diameter, stroke='#000000'):
        radius = diameter * 0.5
        self.require(x-radius,y-radius)
        self.require(x+radius,y+radius)
        self.commands.append(
            '<circle cx="%(x)f" cy="%(y)f" r="%(radius)f" style="fill:none;stroke:%(stroke)s;stroke-width:0.25mm"/>' % locals()
        )

    def line(self, points, color='#000000', width=0.25):
        for x,y in points: self.require(x,y)
        self.commands.append(
            '<polyline points="%s" style="fill:none;stroke:%s;stroke-width:%fmm" />' % (
            ' '.join( '%f,%f'%item for item in points ),
                color,
                width
            )
        )

    def polygon(self, points, color='#000000', width=0.25):
        for x,y in points: self.require(x,y)
        self.commands.append(
            '<polygon points="%s" style="fill:none;stroke:%s;stroke-width:%fmm" />' % (
            ' '.join( '%f,%f'%item for item in points ),
                color,
                width
            )
        )
    
    def profile(self, profile, color='#000000'):
        points = [ ]
        for i, pos in enumerate(profile.pos):
            if i:
                points.append( (profile.low[i], pos) )
            if not i or (profile.low[i] != profile.high[i] and i < len(profile.pos)-1):
                points.append( (profile.high[i], pos) )
        self.line([ ( 0.5*x,-y) for x,y in points ], color)
        self.line([ (-0.5*x,-y) for x,y in points ], color)

    def text(self, x,y, text):
        font_height = 8
        yy = y + font_height * 0.5
        self.require(x,yy-font_height)
        self.require(x+len(text)*font_height*0.8,yy)
        self.commands.append(
            '<text x="%(x)f" y="%(yy)f" fill="#666666" font-size="10" font-family="monospace" xml:space="preserve">%(text)s</text>' % locals()
        )
        return y-font_height




# Numerical techniques for solving 3rd order polynomial spline systems

# The standard representation is the vector of derivatives at s=0,
# with -.5 <= s <= 5.
#
# Thus, \kappa(s) = k0 + k1 s + 1/2 k2 s^2 + 1/6 k3 s^3

from math import *

def eval_cubic(a, b, c, d, x):
    return ((d * x + c) * x + b) * x + a

# integrate over s = [0, 1]
def int_3spiro_poly(ks, n):
    x, y = 0, 0
    th = 0
    ds = 1.0 / n
    th1, th2, th3, th4 = ks[0], .5 * ks[1], (1./6) * ks[2], (1./24) * ks[3]
    k0, k1, k2, k3 = ks[0] * ds, ks[1] * ds, ks[2] * ds, ks[3] * ds
    s = 0
    result = [(x, y)]
    for i in range(n):
        sm = s + 0.5 * ds
        th = sm * eval_cubic(th1, th2, th3, th4, sm)
        cth = cos(th)
        sth = sin(th)

        km0 = ((1./6 * k3 * sm + .5 * k2) * sm + k1) * sm + k0
        km1 = ((.5 * k3 * sm + k2) * sm + k1) * ds
        km2 = (k3 * sm + k2) * ds * ds
        km3 = k3 * ds * ds * ds
        #print km0, km1, km2, km3
        u = 1 - km0 * km0 / 24
        v = km1 / 24

        u = 1 - km0 * km0 / 24 + (km0 ** 4 - 4 * km0 * km2 - 3 * km1 * km1) / 1920
        v = km1 / 24 + (km3 - 6 * km0 * km0 * km1) / 1920

        x += cth * u - sth * v
        y += cth * v + sth * u
        result.append((ds * x, ds * y))

        s += ds

    return result

def integ_chord(k, n = 64):
    ks = (k[0] * .5, k[1] * .25, k[2] * .125, k[3] * .0625)
    xp, yp = int_3spiro_poly(ks, n)[-1]
    ks = (k[0] * -.5, k[1] * .25, k[2] * -.125, k[3] * .0625)
    xm, ym = int_3spiro_poly(ks, n)[-1]
    dx, dy = .5 * (xp + xm), .5 * (yp + ym)
    return hypot(dx, dy), atan2(dy, dx)

# Return th0, th1, k0, k1 for given params
def calc_thk(ks):
    chord, ch_th = integ_chord(ks)
    th0 = ch_th - (-.5 * ks[0] + .125 * ks[1] - 1./48 * ks[2] + 1./384 * ks[3])
    th1 = (.5 * ks[0] + .125 * ks[1] + 1./48 * ks[2] + 1./384 * ks[3]) - ch_th
    k0 = chord * (ks[0] - .5 * ks[1] + .125 * ks[2] - 1./48 * ks[3])
    k1 = chord * (ks[0] + .5 * ks[1] + .125 * ks[2] + 1./48 * ks[3])
    #print '%', (-.5 * ks[0] + .125 * ks[1] - 1./48 * ks[2] + 1./384 * ks[3]), (.5 * ks[0] + .125 * ks[1] + 1./48 * ks[2] + 1./384 * ks[3]), ch_th
    return th0, th1, k0, k1

def calc_k1k2(ks):
    chord, ch_th = integ_chord(ks)
    k1l = chord * chord * (ks[1] - .5 * ks[2] + .125 * ks[3])
    k1r = chord * chord * (ks[1] + .5 * ks[2] + .125 * ks[3])
    k2l = chord * chord * chord * (ks[2] - .5 * ks[3])
    k2r = chord * chord * chord * (ks[2] + .5 * ks[3])
    return k1l, k1r, k2l, k2r
    
def plot(ks):
    ksp = (ks[0] * .5, ks[1] * .25, ks[2] * .125, ks[3] * .0625)
    pside = int_3spiro_poly(ksp, 64)
    ksm = (ks[0] * -.5, ks[1] * .25, ks[2] * -.125, ks[3] * .0625)
    mside = int_3spiro_poly(ksm, 64)
    mside.reverse()
    for i in range(len(mside)):
        mside[i] = (-mside[i][0], -mside[i][1])
    pts = mside + pside[1:]
    cmd = "moveto"
    for j in range(len(pts)):
        x, y = pts[j]
        print 306 + 300 * x, 400 + 300 * y, cmd
        cmd = "lineto"
    print "stroke"
    x, y = pts[0]
    print 306 + 300 * x, 400 + 300 * y, "moveto"
    x, y = pts[-1]
    print 306 + 300 * x, 400 + 300 * y, "lineto .5 setlinewidth stroke"
    print "showpage"

def solve_3spiro(th0, th1, k0, k1):
    ks = [0, 0, 0, 0]
    for i in range(5):
        th0_a, th1_a, k0_a, k1_a = calc_thk(ks)
        dth0 = th0 - th0_a
        dth1 = th1 - th1_a
        dk0 = k0 - k0_a
        dk1 = k1 - k1_a
        ks[0] += (dth0 + dth1) * 1.5 + (dk0 + dk1) * -.25
        ks[1] += (dth1 - dth0) * 15 + (dk0 - dk1) * 1.5
        ks[2] += (dth0 + dth1) * -12 + (dk0 + dk1) * 6
        ks[3] += (dth0 - dth1) * 360 + (dk1 - dk0) * 60
        #print '% ks =', ks
    return ks

def iter_spline(pts, ths, ks):
    pass

def solve_vee():
    kss = []
    for i in range(10):
        kss.append([0, 0, 0, 0])
    thl = [0] * len(kss)
    thr = [0] * len(kss)
    k0l = [0] * len(kss)
    k0r = [0] * len(kss)
    k1l = [0] * len(kss)
    k1r = [0] * len(kss)
    k2l = [0] * len(kss)
    k2r = [0] * len(kss)
    for i in range(10):
        for j in range(len(kss)):
            thl[j], thr[j], k0l[j], k0r[j] = calc_thk(kss[j])
            k0l[j], k1r[j], k2l[j], k2r[j] = calc_k1k2(kss[j])
        for j in range(len(kss) - 1):
            dth = thl[j + 1] + thr[j]
            if j == 5: dth += .1
            dk0 = k0l[j + 1] - k0r[j]
            dk1 = k1l[j + 1] - k1r[j]
            dk2 = k2l[j + 1] - k2r[j]


if __name__ == '__main__':
    k0 = pi * 3
    ks = [0, k0, -2 * k0, 0]
    ks = [0, 0, 0, 0.01]
    #plot(ks)
    thk = calc_thk(ks)
    print '%', thk
    
    ks = solve_3spiro(0, 0, 0, 0.001)
    print '% thk =', calc_thk(ks)
    #plot(ks)
    print '%', ks
    print calc_k1k2(ks)




import sys, math, random, bisect, multiprocessing, signal, time

from nesoni import legion, grace

#def status(*items):
#    """ Display a status string. """
#    string = ' '.join( str(item) for item in items )
#    if sys.stderr.isatty():
#        sys.stderr.write('\r  \x1b[K\x1b[34m' + string + '\x1b[m\r')
#        sys.stderr.flush()


def worker(scorer, fut):
    while True:
        value = legion.coordinator().get_future(fut)
        if value is None: 
            break
        
        item, reply_fut = value
        result = scorer(item)        
        fut = legion.coordinator().new_future()
        legion.coordinator().deliver_future(reply_fut, (result, fut))


def make_update(vecs, initial_accuracy, do_noise):
    do_noise = do_noise or random.random() < 0.1

    #vecs = random.sample(vecs,min(100,len(vecs)))
    
    n = len(vecs)
    m = len(vecs[0])
    
    #mean = [ 
    #    sum([ vec[i] for vec in vecs ])/n 
    #    for i in xrange(m) 
    #    ]

    weight_weight = (1.25+random.random()) / (n**0.5)
    weights = [ 
        random.normalvariate(0.0, weight_weight) 
        for i in xrange(n) 
        ]
    
    offset = (1.0-sum(weights)) / n
    weights = [ weight+offset for weight in weights ]
    
    update = [ 
        sum( vecs[j][i]*weights[j] for j in xrange(n) )
        for i in xrange(m) 
        ]
    
    #update = [ 
    #    sum( 
    #        (vecs[j][i]-mean[i])*weights[j] 
    #        for j in xrange(n) 
    #        ) + mean[i]
    #    for i in xrange(m) 
    #    ]

    if do_noise:
        extra = random.random() * initial_accuracy
        update = [ 
            val+random.normalvariate(0.0, extra) 
            for val in update 
            ]
    
    return update


def improve(comment, constrainer, scorer, start_x, ftol=1e-4, xtol=1e-6, initial_accuracy=0.01, monitor = lambda x,y: None):
    pool_size = legion.coordinator().get_cores()
    
    worker_futs = [ legion.coordinator().new_future() for i in xrange(pool_size) ]
    reply_futs = [ ]
    
    workers = [
        legion.future(worker,scorer,fut)
        for fut in worker_futs 
        ]
    
    last_t = 0.0
    try:
        best = start_x
        c_score = constrainer(best)
        if c_score:
            best_score = (c_score, 0.0)
        else:
            best_score = (0.0, scorer(best))
        
        n_good = 0
        n_real = 0
        i = 0
        jobs = [ ]
        
        pool_size = int(len(best)*5)
        print len(best),'parameters, pool size', pool_size

        currents = [ (best, best_score) ]
        
        done = False
        while not done or reply_futs:
            t = time.time()
            if t > last_t+20.0:
                def rep(x): 
                    if x[0]: return 'C%.6f' % x[0]
                    return '%.6f' % x[1]
                grace.status('%s %s %d %d %d %d %s'%(rep(best_score), rep(max(item[1] for item in currents)), len(currents), n_good, n_real, i, comment))
                if best_score[0] == 0:
                    monitor(best, [ item[0] for item in currents ])
                last_t = time.time()
            
            have_score = False
            
            if not done and worker_futs:
                new = make_update([item[0] for item in currents], initial_accuracy, len(currents) < pool_size)
                
                c_score = constrainer(new)
                if c_score:
                    have_score = True
                    new_score = (c_score, 0.0)
                else:
                    reply_fut = legion.coordinator().new_future()
                    worker_fut = worker_futs.pop(0)                    
                    legion.coordinator().deliver_future(worker_fut, (new, reply_fut))
                    reply_futs.append( (new, reply_fut) )
            
            if not have_score:
                if not reply_futs or (not done and worker_futs):
                    continue
                new, reply_fut = reply_futs.pop(0)
                new_score, worker_fut = legion.coordinator().get_future(reply_fut)
                new_score = (0.0, new_score)
                worker_futs.append(worker_fut)
            
            if new_score[0] == 0.0:
                n_real += 1

            l = sorted( item[1][1] for item in currents )
            if pool_size < len(l):
                c = l[pool_size]
            else:
                c = 1e30
            cutoff = (best_score[0], c)
            
            if new_score <= cutoff:
                currents = [ item for item in currents if item[1] <= cutoff ]
                currents.append((new,new_score))
                
                n_good += 1
            
                if new_score < best_score:
                    best_score = new_score
                    best = new
            
            if not best_score[0]:
                xspan = 0.0
                for i in xrange(len(start_x)):
                    xspan = max(xspan,
                        max(item[0][i] for item in currents) -
                          min(item[0][i] for item in currents)
                        )
                
                fspan = (max(item[1] for item in currents)[1]-best_score[1]) 
                
                if xspan < xtol or (n_good >= 5000 and fspan < ftol):
                    done = True
            i += 1
        
        grace.status('')
        print '%s %.5f\n' % (comment, best_score[1])
        
    finally:
        #pool.terminate()
        pass
    
    while worker_futs:
        fut = worker_futs.pop(0)
        legion.coordinator().deliver_future(fut, None)
    
    for item in workers:
        item()
    
    return best
        



import sys, os, math

import cpp

import shape

CODE = r"""
#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
#include <CGAL/Simple_homogeneous.h>

#include <CGAL/Nef_polyhedron_3.h>
#include <CGAL/Polyhedron_incremental_builder_3.h>
#include <CGAL/Polyhedron_3.h>
#include <CGAL/minkowski_sum_3.h>

#include <CGAL/Polygon_2.h>
#include <CGAL/Polygon_with_holes_2.h>
#include <CGAL/Polygon_set_2.h>
#include <CGAL/Boolean_set_operations_2.h>
#include <CGAL/minkowski_sum_2.h>
#include <CGAL/Small_side_angle_bisector_decomposition_2.h>
#include <CGAL/connect_holes.h>

#include <vector>
#include <iostream>
#include <sstream>

//typedef CGAL::Exact_predicates_exact_constructions_kernel Kernel;
//typedef CGAL::Extended_homogeneous<CGAL::Gmpz>            Kernel;
//typedef CGAL::Cartesian<CGAL::Gmpq>                       Kernel;
//typedef CGAL::Homogeneous<CGAL::Gmpz>                     Kernel;
typedef CGAL::Simple_homogeneous<CGAL::Gmpz>                Kernel;

typedef Kernel::Point_3                                   Point_3;
typedef Kernel::Vector_3                                  Vector_3;
typedef CGAL::Polyhedron_3<Kernel>                        Polyhedron;
typedef CGAL::Nef_polyhedron_3<Kernel>                    Nef_polyhedron;
typedef Polyhedron::HalfedgeDS                            HalfedgeDS;
typedef CGAL::Aff_transformation_3<Kernel>                Aff_transformation_3;

typedef Kernel::Point_2                                   Point_2;
typedef Kernel::Vector_2                                  Vector_2;
typedef CGAL::Polygon_2<Kernel>                           Polygon_2;
typedef CGAL::Polygon_with_holes_2<Kernel>                Polygon_with_holes_2;
typedef CGAL::Polygon_set_2<Kernel>                       Polygon_set_2;
typedef CGAL::Aff_transformation_2<Kernel>                Aff_transformation_2;
typedef CGAL::Small_side_angle_bisector_decomposition_2<Kernel> Decomposition_strategy;

class Build_polyhedron : public CGAL::Modifier_base<HalfedgeDS> {
public:
    std::vector<Kernel::Point_3> points;
    std::vector< std::vector<int> > facets;

    Build_polyhedron() {}
    void operator()( HalfedgeDS& hds) {
        CGAL::Polyhedron_incremental_builder_3<HalfedgeDS> B( hds, true);
         
        B.begin_surface( points.size(), facets.size() );
    
        for(std::vector<Kernel::Point_3>::iterator it = points.begin();
            it < points.end();
            it++) {
            B.add_vertex(*it);
        }
        
        for(std::vector< std::vector<int> >::iterator it = facets.begin();
            it < facets.end();
            it++) {
            B.begin_facet();
            for(std::vector<int>::iterator it1 = it->begin();
                it1 < it->end();
                it1++) {
                B.add_vertex_to_facet(*it1);
            }
            B.end_facet();
        }
        
        B.end_surface();
    }
};

"""

CMAKE_CODE = r"""

find_package(CGAL REQUIRED)

include(${CGAL_USE_FILE})

"""

M = cpp.Module(os.path.join(os.path.expanduser('~'),'.demakein'), CODE, CMAKE_CODE)

def int_round(value):
    return int(math.floor(value+0.5))

def rotation_matrix(x,y,z,angle):
    length = math.sqrt(x*x+y*y+z*z)
    x = x/length
    y = y/length
    z = z/length
    angle = angle * (math.pi/180.0)
    c = math.cos(angle)
    s = math.sin(angle)
    return [
        [ c+x*x*(1-c),   x*y*(1-c)-z*s, x*z*(1-c)+y*s ],
        [ y*x*(1-c)+z*s, c+y*y*(1-c),   y*z*(1-c)-x*s ],
        [ z*x*(1-c)-y*s, z*y*(1-c)+x*s, c+z*z*(1-c)   ],
    ]

def transform_point_3(matrix, point):
    return tuple(
        sum( matrix[i][j] * point[j] for j in (0,1,2) )
        for i in (0,1,2)
    )

def extent_3(points):
    xs = []
    ys = []
    zs = []
    for point in points:
        xs.append(point[0])
        ys.append(point[1])
        zs.append(point[2])
    return shape.Limits(min(xs),max(xs),min(ys),max(ys),min(zs),max(zs))


class Shape(object):
    """ Wrapper around Nef_polyhedron """
    
    def __init__(self, nef):
        self.nef = nef

    def copy(self):
        return Shape(M.new('Nef_polyhedron(a)',a=self.nef))

    def extent(self):
        xs = []
        ys = []
        zs = []
        for tri in self.iter_triangles(): #Possibly slow?
            for point in tri:
                xs.append(point[0])
                ys.append(point[1])
                zs.append(point[2])
        #for vertex in M.iterate('a.vertices_begin()','a.vertices_end()',a=self.nef):
        #    point = M('a.point()',a=vertex)
        #    xs.append(M('CGAL::to_double(a.x())',a=point))
        #    ys.append(M('CGAL::to_double(a.y())',a=point))
        #    zs.append(M('CGAL::to_double(a.z())',a=point))
        return shape.Limits(min(xs),max(xs),min(ys),max(ys),min(zs),max(zs))
    
    def size(self):
        xmin,xmax,ymin,ymax,zmin,zmax = self.extent()
        return xmax-xmin,ymax-ymin,zmax-zmin

    def iter_triangles(self):
        point_cache = { }
        def get_cache(p):
            if p not in point_cache:
                point_cache[p] = p
            return point_cache[p]
        
        polyhedron = M.new('Polyhedron()')
        #M('a.interior().convert_to_polyhedron(b)',a=self.nef,b=polyhedron)
        M('a.convert_to_polyhedron(b)',a=self.nef,b=polyhedron)
        
        #assert M('(int)a.is_valid(true, 3)',a=polyhedron)
        
        triangles = [ ]
        for item in M.iterate('a.facets_begin()','a.facets_end()',a=polyhedron):
            verts = [ ]
            for item2 in M.circulate('a.facet_begin()',a=item):
                #point = M('a.vertex()->point()',a=item2)
                verts.append(get_cache((
                     M('CGAL::to_double(a.vertex()->point().x())',a=item2),
                     M('CGAL::to_double(a.vertex()->point().y())',a=item2),
                     M('CGAL::to_double(a.vertex()->point().z())',a=item2),
                )))
                
            assert len(verts) == 3
            yield tuple(verts)
    
    def triangles(self):
        return list(self.iter_triangles())

    def save(self, filename):
        print filename,
        sys.stdout.flush()
        with open(filename,'wb') as f:
            print >> f, 'solid'
            n = 0
            for tri in self.iter_triangles():
                n += 1
                print >> f, 'facet normal 0 0 0'
                print >> f, 'outer loop'
                for vert in tri:
                    print >> f, 'vertex %f %f %f' % tuple(vert)
                print >> f, 'endloop'
                print >> f, 'endfacet'
            print >> f, 'endsolid'
        print n, 'triangles'

    def remove(self, other):
        M.do('a -= b',a=self.nef,b=other.nef)

    def add(self, other):
        M.do('a += b',a=self.nef,b=other.nef)

    def clip(self, other):
        M.do('a *= b',a=self.nef,b=other.nef)

    def inverse(self):
        return Shape(M('!a',a=self.nef))

    def minkowski_sum(self, other):
        return Shape(M('CGAL::minkowski_sum_3(a,b)',a=self.nef,b=other.nef))

    def rotate(self, x,y,z,angle, accuracy=1<<16):
        if not angle: return    
        rot = rotation_matrix(x,y,z,angle)        
        transform = M.call('Aff_transformation_3',
            *[ int_round(item*accuracy) for row in rot for item in row ] + [ accuracy ]
        )
        M.do('a.transform(b)',a=self.nef, b=transform)
        
        # Rotation of a nef appears to be extremely memory hungry
        # TODO: retain exact representation
        #
        #verts = [ ]
        #vert_index = { }
        #tris = [ ]
        #for tri in self.iter_triangles():
        #    tri_verts = [ ]
        #    for point in tri:
        #        if point not in vert_index:
        #            vert_index[point] = len(verts)
        #            verts.append(point)
        #        tri_verts.append(vert_index[point])
        #    tris.append(tri_verts)
        #
        #self.nef = create([ shape.transform_point_3(rot,item) for item in verts ], tris).nef
        
    
    def move(self, x,y,z, accuracy=1<<16):
        transform = M('Aff_transformation_3(CGAL::TRANSLATION,Vector_3(a,b,c,d))',
            a=int_round(x*accuracy), b=int_round(y*accuracy), c=int_round(z*accuracy), d=accuracy
        )
        M.do('a.transform(b)',a=self.nef, b=transform)

    def position_nicely(self):
        e = self.extent()
        self.move(-0.5*(e.xmin+e.xmax),-0.5*(e.ymin+e.ymax),-e.zmin)

    def mask(self, res):
        import mask
        
        lines = [ ]
        
        for points in self.triangles():
            points = [ (x*res,y*res) for x,y,z in points ]
            points = points + [ points[0] ]
            
            area = 0.0
            for i in range(len(points)-1):
                area += (
                    (points[i+1][0]-points[i][0])*
                    (points[i+1][1]+points[i][1])*0.5
                )
            if area > 0.0: 
                points = points[::-1]
            for i in range(len(points)-1):
                lines.append( points[i]+points[i+1] )
            
        return mask.make_mask(lines)
    
    def polygon_mask(self):
        triangles = self.triangles()
        
        things = [ create_polygon_2([ (x,y) for x,y,z in triangle ]) 
                   for triangle in triangles ]

        if not things:        
            return empty_shape_2()

        while len(things) > 1:
            item = things.pop(-1)
            things[len(things)//2].add(item)
        return things[0]
        
        
        #result = empty_shape_2()
        #for triangle in self.triangles():
        #    result.add( create_polygon_2([ (x,y) for x,y,z in triangle ]) )
        #return result
        
    
    #def show(self):
    #    app = M.new('QApplication(argc, argv)',argc=0,argv=0)
    #    widget = M('new CGAL::Qt_widget_Nef_3<Nef_polyhedron_3>(a)',a=self.nef)
    #    M.do('a.setMainWidget(b)',a=app,b=widget)
    #    M.do('a.show()',a=widget)
    #    return M('a.exec()',a=app)

def empty_shape():
    return Shape(M.new('Nef_polyhedron'))

def create(verts, tris, name=None, accuracy=1<<16):
    polyhedron = M.new('Polyhedron()')
    
    builder = M.new('Build_polyhedron()')
    
    for tri in tris:
        assert len(tri) == 3
    
    #seen = set()
    vert_hash = { }
    vert_remap = [ ]
    n_vert_merges = 0
    for vert in verts:
        [x,y,z] = [ int_round(item*accuracy) for item in vert ]
        #assert (x,y,z) not in seen
        #seen.add((x,y,z))
        if (x,y,z) not in vert_hash:
            point = M.new('Point_3(a,b,c,d)',a=x,b=y,c=z,d=accuracy)
            M.do('a.points.push_back(b)',a=builder,b=point)
            vert_hash[(x,y,z)] = len(vert_hash)
        else:
            n_vert_merges += 1
        vert_remap.append(vert_hash[(x,y,z)])
    
    n_tri_losses = 0
    for tri in tris:
        #assert tri[0] != tri[1] and tri[0] != tri[2] and tri[1] != tri[2]
        tri = [ vert_remap[i] for i in tri ]
        if tri[0] == tri[1] or tri[0] == tri[2] or tri[1] == tri[2]:
            n_tri_losses += 1
            continue
        
        vec = M.new('std::vector<int>()')
        for item in tri: 
            M.do('a.push_back(b)',a=vec,b=item)
        M.do('a.facets.push_back(b)',a=builder,b=vec)
    
    if n_vert_merges or n_tri_losses:
        print 'Verts merged: %d of %d, triangles discarded: %d of %d' % (n_vert_merges, len(verts), n_tri_losses, len(tris))
    
    M.do('a.delegate(b)',a=polyhedron,b=builder)
    
    assert M('a.is_closed()',a=polyhedron), 'Polyhedron is not closed'        

    nef = M.new('Nef_polyhedron(a)',a=polyhedron)    
    
    assert M('a.is_valid()',a=nef)
    assert M('a.is_simple()',a=nef)    
    
    return Shape(nef)

def create_polygon_3(verts, accuracy=1<<16):
    vec = M.new('std::vector<Point_3>')

    for vert in verts:
        [x,y,z] = [ int_round(item*accuracy) for item in vert ]
        M.do('a.push_back(Point_3(b,c,d,e))',a=vec,b=x,c=y,d=z,e=accuracy)
    
    return Shape(M.new('Nef_polyhedron(a.begin(),a.end())',a=vec))

#Needs more complicated representation
#def create_halfspace():
#    return Shape(M.new('Nef_polyhedron(Nef_polyhedron::Plane_3(0,0,1,0))'))

def read_stl(filename):
    with open(filename,'rU') as f:
        verts = [ ]
        for line in f:
            parts = line.rstrip().split()
            if parts[0] == 'vertex':
                verts.append((float(parts[1]),float(parts[2]),float(parts[3])))
            elif parts[0] == 'endloop':
                assert len(verts) == 3
                yield verts
                verts = [ ]
        assert not verts

def load_stl(filename):
    verts = [ ]
    vert_index = { }
    tris = [ ]
    for triangle in read_stl(filename):
        tri = [ ]
        for point in triangle:
            if point not in vert_index:
                vert_index[point] = len(verts)
                verts.append(point)
            tri.append(vert_index[point])
        assert len(set(tri)) == 3, tri
        tris.append(tri)
    
    return create(verts, tris)



def show_only(*shapes):
    pass




class Shape_2(object):
    def __init__(self, pset):
        self.pset = pset

    def copy(self):
        return Shape_2(M.new('Polygon_set_2(a)',a=self.pset))
    
    def _iter_polygons_with_holes(self):
        phole_vec = M.new('std::vector<Polygon_with_holes_2>')
        M.do('a.polygons_with_holes(std::back_inserter(b))', a=self.pset,b=phole_vec)        
        for phole in M.iterate('a.begin()','a.end()',a=phole_vec):
            yield phole
            
    def loops(self, holes=True):
        def point_decode(point):
            return (
                M('CGAL::to_double(a.x())',a=point),
                M('CGAL::to_double(a.y())',a=point)
            )
        def polygon_decode(poly):
            return shape.Loop([ point_decode(item) for item in M.iterate('a.vertices_begin()','a.vertices_end()',a=poly) ])
    
        loops = [ ]
    
        for phole in self._iter_polygons_with_holes():
            assert not M('a.is_unbounded()',a=phole)
            loops.append( polygon_decode( M('a.outer_boundary()',a=phole) ) )
            if holes:
                loops.extend( polygon_decode(item) for item in  M.iterate('a.holes_begin()','a.holes_end()',a=phole) )
        return loops

    def loop(self, holes=True):
        [ result ] = self.loops(holes)
        return result

    def extent(self):
        xs = [ ]
        ys = [ ]
        for loop in self.loops():
            for point in loop:
                xs.append(point[0])
                ys.append(point[1])
        return shape.Limits_2(
            min(xs), max(xs),
            min(ys), max(ys),
        )
    
    def to_3(self):
        #TESTME!
   
        def polygon_recode(poly):
            vec = M.new('std::vector<Point_3>')
            for point_3 in M.iterate('a.vertices_begin()','a.vertices_end()',a=poly):
                M.do('a.push_back(Point_3(b.x(),b.y(),0))',a=vec,b=point_3)
            return Shape(M.new('Nef_polyhedron(a.begin(),a.end())',a=vec))
        
        #result = empty_shape()
        result = None
        for phole in self._iter_polygons_with_holes():
            phole_3 = polygon_recode( M('a.outer_boundary()',a=phole) )
            for hole in M.iterate('a.holes_begin()','a.holes_end()',a=phole):
                phole_3.remove( polygon_recode(hole) )
            if result is None:
                result = phole_3
            else:
                result.add(phole_3)
        
        assert result is not None #CGAL gets explodey with empty shapes
            
        return result
        
    def remove(self, other):
        M.do('a.difference(b)',a=self.pset,b=other.pset)

    def add(self, other):
        M.do('a.join(b)',a=self.pset,b=other.pset)

    def clip(self, other):
        M.do('a.intersection(b)',a=self.pset,b=other.pset)

    def invert(self):
        M.do('a.complement()',a=self.pset)
    
    def orientation(self):
        return M('(int)a.orientation()',a=self.pset)
    
    def is_empty(self):
        return M('a.is_empty()',a=self.pset)


    def _reduce_things(self, things):
        if not things:
            return empty_shape_2()
        while len(things) > 1:
            item = things.pop(-1)
            things[len(things)//2].add(item)
        return things[0]

    def move(self, x,y, accuracy=1<<16):
        #transform = M('Aff_transformation_2(CGAL::TRANSLATION,Vector_2(a,b,c))',
        #    a=int_round(x*accuracy), b=int_round(y*accuracy), c=accuracy
        #)
        offset = M('Vector_2(a,b,c)',
            a=int_round(x*accuracy), b=int_round(y*accuracy), c=accuracy
        )
        def transform(polygon):
            vec = M.new('std::vector<Point_2>')
            for point in M.iterate('a.vertices_begin()','a.vertices_end()',a=polygon):
                M.do('a.push_back(b+c)',a=vec,b=point,c=offset)
            return M.new('Polygon_2(a.begin(),a.end())',a=vec)
        
        #M.do('a.transform(b)',a=self.pset, b=transform)
        things = [ ]
        for phole in self._iter_polygons_with_holes():
            outside = transform( M('a.outer_boundary()',a=phole) )
            insides = M.new('std::vector<Polygon_2>')
            for inside in M.iterate('a.holes_begin()','a.holes_end()',a=phole):
                inside = transform(inside)
                M.do('a.push_back(b)',a=insides,b=inside)
            thing = M.new('Polygon_set_2(Polygon_with_holes_2(a,b.begin(),b.end()))',a=outside,b=insides)
            things.append(thing)       
            
        self.pset = self._reduce_things(things)

    def intersects(self, other):
        temp = self.copy()
        temp.clip(other)
        return not temp.is_empty()
    
        #return M('CGAL::do_intersect(a,b)',a=self.pset,b=other.pset)
        #bholes = list(other._iter_polygons_with_holes())
        #for ahole in self._iter_polygons_with_holes():
        #    for bhole in bholes:
        #        if M('CGAL::do_intersect(a,b)',a=ahole,b=bhole):
        #            return True
        #return False

    def minkowski_sum(self, other):
        things = [ ]
        my_polys = map(_connect_holes,self._iter_polygons_with_holes())
        other_polys = map(_connect_holes,other._iter_polygons_with_holes())
        for a in my_polys:
            for b in other_polys:
                things.append(Shape_2(
                    M.new(
                        'Polygon_set_2(CGAL::minkowski_sum_2(a,b,Decomposition_strategy()))',
                        a=a,b=b)
                    ))
        return self._reduce_things(things)

    def erosion(self, other):
        ext1 = self.extent()
        ext2 = other.extent()
        #Generous margin
        r = max(abs(item) for item in list(ext1)+list(ext2))
        inner_loop = shape.square(r*3)
        outer_loop = shape.square(r*5)
        
        negative = outer_loop.shape_2()
        negative.remove(self)
        dilation = negative.minkowski_sum(other)
        result = inner_loop.shape_2()
        result.remove(dilation)
        return result
    
    def offset_curve(self, amount, quality=None):
        """ +ve dilation
            -ve erosion """
        if amount == 0.0:
            return self.copy()
        elif amount > 0.0:
            return self.minkowski_sum(shape.circle(amount*2.0, quality).shape_2())
        else:
            return self.erosion(shape.circle(amount*-2.0, quality).shape_2())    
        

def _connect_holes(poly_with_holes):
    """ Converts a polygon_with_holes to a polygon """
    vec = M.new('std::vector<Point_2>')
    M.do('CGAL::connect_holes(a,std::back_inserter(b))',a=poly_with_holes,b=vec)
    poly = M.new('Polygon_2(a.begin(),a.end())',a=vec)
    return poly


def empty_shape_2():
    return Shape_2(M.new('Polygon_set_2'))


def create_polygon_2(verts, accuracy=1<<16):
    vec = M.new('std::vector<Point_2>')

    for vert in verts:
        [x,y] = [ int_round(item*accuracy) for item in vert ]
        M.do('a.push_back(Point_2(b,c,d))',a=vec,b=x,c=y,d=accuracy)
    
    poly = M.new('Polygon_2(a.begin(),a.end())',a=vec)
    
    orientation = M('(int)a.orientation()',a=poly)
    
    if orientation == 0:
        return empty_shape_2()
        
    if orientation < 0:
        M('a.reverse_orientation()',a=poly)
    
    return Shape_2(M.new('Polygon_set_2(a)',a=poly))
    



def main(func, *args, **kwargs):
    #return M.run(func,sys.argv[1:],*args,**kwargs)
    return func(sys.argv[1:],*args,**kwargs)


if __name__ == '__main__':
    def run():
        #print module.as_reference('new int(42)').__dict__
        #print 'hello'
        #return
    
        polyhedron = M.new('Polyhedron()')
    
        #builder = module('Build_polyhedron()')
        #
        #for p in [(1,0,0),(0,1,0),(0,0,1)]:
        #    point = module.call('Kernel::Point_3',*p)
        #    module('a.points.push_back(b)',a=builder,b=point)
        #
        #for f in [(0,1,2)]:
        #    vec = module('std::vector<int>()')
        #    for item in f: 
        #        module('a.push_back(b)',a=vec,b=item)
        #    module('a.facets.push_back(b)',a=builder,b=vec)
        #
        #module('a.delegate(b)',a=polyhedron,b=builder)
    
        M('a.make_tetrahedron(Point_3(0,0,0),Point_3(1,0,0),Point_3(0,1,0),Point_3(0,0,1))',a=polyhedron)
    
        print M('a.size_of_vertices()',a=polyhedron)
        print M('a.size_of_halfedges()',a=polyhedron)
        print M('a.size_of_facets()',a=polyhedron)
        
        print 'Closed:', M('(int)a.is_closed()',a=polyhedron)
        
        nef = M.new('Nef_polyhedron(a)',a=polyhedron)
        
        print 'Simple:', M('(int)a.is_simple()',a=nef)
        
        polyhedron2 = M.new('Polyhedron()')
        M('a.convert_to_polyhedron(b)',a=nef,b=polyhedron2)
    
        print 'Closed:', M('(int)a.is_closed()',a=polyhedron2)
        
        #i = module('a.facets_begin()',a=polyhedron2)
        #end = module('a.facets_end()',a=polyhedron2)
        #while module('a<b',a=i,b=end):
        #    value = module('*a',a=i)
        #    print 'boop'
        #    module('a++',a=i)
            
        for item in M.iterate('a.facets_begin()','a.facets_end()',a=polyhedron2):
            print 'boop'
            for item2 in M.circulate('a.facet_begin()',a=item):
                print M('a.vertex()->point()',a=item2)
    
        print 'lovely'
    
    #M.run(run)
    run()
    




from math import *

def plot_elastica(a, c):
    s = 500
    cmd = 'moveto'
    dx = .001
    x, y = 0, 0
    if c * c > 2 * a * a:
        g = sqrt(c * c - 2 * a * a)
        x = g + .01
    if c == 0:
        x = .001
    try:
        for i in range(1000):
            print 6 + s * x, 200 + s * y, cmd
            cmd = 'lineto'
            x += dx
            if 1 and c * c > 2 * a * a:
                print (c * c - x * x) * (x * x - g * g)
                dy = dx * (x * x - .5 * c * c - .5 * g * g) / sqrt((c * c - x * x) * (x * x - g * g))
            else:
                dy = dx * (a * a - c * c + x * x)/sqrt((c * c - x * x) * (2 * a * a - c * c + x * x))
            y += dy
    except ValueError, e:
        pass
    print 'stroke'

plot_elastica(1, 0)
print 'showpage'



VERSION = '0.13'

from .design_flute import Design_pflute, Design_folk_flute
from .make_flute import Make_flute
from .make_cork import Make_cork

from .design_shawm import Design_shawm, Design_folk_shawm
from .make_shawm import Make_shawm
from .make_mouthpiece import Make_mouthpiece
from .make_bauble import Make_bauble
from .make_windcap import Make_windcap

from .design_whistle import Design_folk_whistle, Design_recorder, Design_three_hole_whistle
from .make_whistle import Make_whistle

from .make_panpipe import Make_panpipe

from .make_reed import Make_reed, Make_reed_shaper

from .tune import Tune

from .all import All

def main():
    """ Command line interface. """
    import nesoni    
    nesoni.run_toolbox([
            'Demakein '+VERSION,
            'Flutes',
            Design_pflute, 
            Design_folk_flute, 
            Make_flute,
            Make_cork,
            
            'Whistles',
            Design_folk_whistle,
            Design_recorder,
            Design_three_hole_whistle,
            Make_whistle,
            
            'Shawms',
            Design_shawm,
            Design_folk_shawm,
            Make_shawm,
            Make_mouthpiece,
            Make_windcap,
            Make_bauble,            
            #Make_reed,
            Make_reed_shaper,
            
            'Panpipes',        
            Make_panpipe,
            
            'Utilities',
            Tune,
            
            #'Everything',        
            #All,
            #'"demakein all:" uses the nesoni make system, see flags below.',
        ],
        show_make_flags=False,
        )







import sys, os, math

sys.path.insert(0, os.path.normpath(os.path.join(__file__,'..','..')))

import demakein
from demakein import shape, geom, make, profile, pack

import nesoni
from nesoni import config

@config.help("""\
Make the head of a whistle instrument.
""")
@config.Float_flag('bore')
@config.Float_flag('outside')
@config.Float_flag('gap_width')
@config.Float_flag('gap_length')
class Make_whistle_head(make.Make):
    bore = 15.0
    outside = 21.0
    
    gap_width = 0.6 #0.5 #0.666 #Formerly 0.5
    gap_length = 0.25 #0.3 #0.4 #Formerly 0.25
    # (as a proportion of bore diameter)

    def effective_gap_diameter(self):
        area = (self.bore*self.gap_length) * (self.bore*self.gap_width)
        return math.sqrt(area/math.pi) * 2.0
    
    def effective_gap_height(self):
        # TODO: further adjustment
        return (self.outside-self.bore) * 0.5

    def construct(self):
        bore = self.bore
        outside = self.outside
        #outside = bore * 1.4
        
        bore_length = bore*2.0        
        gap_length = bore * self.gap_length
        windcutter_length = bore * 1.0
        
        airway_length = bore*1.5

        z_min = -bore_length        
        z_gap_0 = z_min + bore_length - gap_length
        z_gap_1 = z_min + bore_length
        z_windcutter_0 = z_gap_0 - windcutter_length
        z_windcutter_1 = z_gap_0
        z_airway_0 = z_gap_1
        z_airway_1 = z_airway_0 + airway_length
        z_max = z_airway_1
        
        #airway_xsize = bore * 0.15
        #airway_xsize = (outside-bore) * 0.4
        airway_xsize = 2.0 #Constant!
        
        airway_ysize = bore * self.gap_width
        
        #print 'Note mill bit diameter must be less than: %.1fmm' % (airway_xsize*2.0)
        
        windcutter_rounding = bore * 0.1
        windcutter_ysize = airway_ysize + windcutter_rounding
        windcutter_lip = bore * 0.02

        airway_x_low  = bore*0.5-airway_xsize*0.5+windcutter_lip*0.5
        airway_x_high = bore*0.5+airway_xsize*0.5+windcutter_lip*0.5
        
        airway_line_0 = profile.make_profile(
            [(z_airway_0, airway_x_low),
             (z_airway_1, airway_x_low)])

        airway_line_1 = profile.make_profile(
            [(z_airway_0, airway_x_high),
             (z_airway_1, airway_x_high)])
        
        windcutter_line = profile.make_profile(
            [(z_windcutter_0, outside*0.5),
             (z_windcutter_1, bore*0.5+windcutter_lip)])
        
        undercutter_line = profile.make_profile(
            [(z_windcutter_0, bore*0.35),
             (z_windcutter_1, bore*0.5)])
        
        body = shape.extrude_profile(
            profile.make_profile([(z_min,outside),(z_max,outside)]),
            )
        
        bore_space = shape.extrude_profile(
            profile.make_profile([(z_min,bore),(z_gap_1,bore)]),
            )
        #body.remove(bore_space)
        space = bore_space.copy()
         
        windcutter_space = shape.extrude_profile(
            windcutter_line.clipped(z_windcutter_0-1.0,z_gap_1),
            cross_section = lambda x:
                shape.rounded_rectangle(
                    x, x+bore,
                    windcutter_ysize*-0.5,windcutter_ysize*0.5,
                    windcutter_rounding)
            )
        body.remove(windcutter_space)
        #space.add(windcutter_space)
        
        undercutter_space = shape.extrude_profile(
            undercutter_line.clipped(z_windcutter_0,z_gap_1),
            cross_section = lambda x:
#                shape.halfrounded_rectangle(
                shape.rectangle(
                    x-airway_xsize, x,
                    airway_ysize*-0.5,airway_ysize*0.5)
            )
        #body.remove(undercutter_space)
        space.add(undercutter_space)
               
        airway_space = shape.extrude_profile(
            airway_line_0.clipped(z_gap_0,z_airway_1+ airway_xsize*2), #Make longer, for milling inside
            airway_line_1.clipped(z_gap_0,z_airway_1+ airway_xsize*2),
            cross_section = lambda x0, x1:
                shape.rectangle(
                    x0, x1,
                    airway_ysize*-0.5,airway_ysize*0.5)
            )
        body.remove(airway_space)
        space.add(airway_space)
        
        gap_space = shape.block(
            0.0, bore,
            airway_ysize*-0.5,airway_ysize*0.5,
            z_gap_0,z_gap_1
            )
        body.remove(gap_space)
        #space.add(gap_space)
        
        cutaway_diameter = outside*1.5
        cutaway_space = shape.extrude_profile(
            profile.make_profile([(-outside*0.51, cutaway_diameter),(outside*0.51,cutaway_diameter)]))
        cutaway_space.rotate(1,0,0, 90)
        cutaway_space.move(-cutaway_diameter*0.5+bore*0.5-(outside*0.5-bore*0.5),0,z_max)
        #cutaway_space.clip(shape.block(-outside*0.51,outside*0.51,-outside*0.51,outside*0.51,z_min,z_max))
        
        body.remove(cutaway_space)
        space.add(cutaway_space)
                
        #self.save(body, 'whistle')
        
        #assert airway_xsize_0 == airway_xsize_1
        #jaw_clipper = shape.extrude_profile(
        #    profile.make_profile([(-outside,0),(outside,0)]),
        #    cross_section=lambda foo: shape.rounded_rectangle(
        #        z_gap_0, z_max+gap_length,
        #        -outside, airway_line_0(z_airway_0),
        #        gap_length*2.0)
        #    )
        #jaw_clipper.rotate(1,0,0,90)
        #jaw_clipper.rotate(0,1,0,-90)
        #jaw_clipper.rotate(0,0,1,180)
        
        #jaw_clipper = shape.extrude_profile(
        #    airway_line_0.clipped(z_airway_0,z_airway_1+1.0),
        #    airway_line_1.clipped(z_airway_0,z_airway_1+1.0),
        #    cross_section = lambda x0, x1:
        #        shape.rounded_rectangle(
        #            -x0*0.8, #(Can't be larger than bore) 
        #            x1 * 0.995, #So as to not leave a little fuzz
        #            airway_ysize*-0.5,airway_ysize*0.5,
        #            (x1-x0)*2.0)
        #    )
        
        d = airway_x_low*2        
        jaw_clipper = shape.extrude_profile(
            profile.make_profile([(-outside*0.5-10, d),(outside*0.5+10, d)]),
            cross_section = lambda d:
                shape.halfrounded_rectangle(
                    -0.001, airway_x_low*1.001,
                    z_airway_0-d*0.5,z_max+d*0.5)
            )
        jaw_clipper.rotate(1,0,0, 90)
        
        #jaw = jaw_clipper.copy()
        #jaw.remove(space)
        #jaw.clip(body)   
        #self.save(jaw, 'jaw')
        
        #head = body.copy()
        #head.remove(jaw)
        #self.save(head, 'head')
        
        return body, space, jaw_clipper
    
    def run(self):
        body, space, jaw_clipper = self.construct()
        
        body.remove(space)
        self.save(body, 'whistle')
        
        jaw = body.copy()
        jaw.clip(jaw_clipper)
        body.remove(jaw_clipper)
        self.save(body, 'head')
        self.save(jaw, 'jaw')


@config.help("""\
Produce 3D models using the output of "demakein design-*-whistle:".
""")
class Make_whistle(make.Make_millable_instrument):
    def run(self):
        working = self.working
        designer = working.designer
        spec = working.spec
        workspace = self.get_workspace()
        
        whistle_maker = designer.get_whistle_maker()
        whistle_outer, whistle_inner, whistle_jaw_clipper = whistle_maker.construct()
        
        whistle_outer.move(0,0,spec.length)
        whistle_inner.move(0,0,spec.length)
        whistle_jaw_clipper.move(0,0,spec.length)
        whistle_outer.rotate(0,0,1,90)
        whistle_inner.rotate(0,0,1,90)
        whistle_jaw_clipper.rotate(0,0,1,90)
        
        n_holes = len(spec.hole_diameters)
        
        self.make_instrument(
             inner_profile = spec.inner.clipped(-50, spec.length),
             outer_profile = spec.outer.clipped(0, spec.length-whistle_maker.bore*1.5), #Don't paint over windcutter
             hole_positions = spec.hole_positions,
             hole_diameters = spec.hole_diameters,
             hole_vert_angles = spec.hole_angles,
             hole_horiz_angles = designer.horiz_angles,
             xpad = [0.0]*n_holes,
             ypad = [0.0]*n_holes,
             with_fingerpad = [True]*n_holes,
             outside_extras = [ whistle_outer ],
             bore_extras = [ whistle_inner ],
             )
        
        true_length = self.working.instrument.extent().zmax

        #cut0 = spec.hole_positions[0] + spec.hole_diameters[0]*0.75
        #cut0 += spec.inner(cut0)*0.5
        #cut1 = spec.hole_positions[designer.mid_cut] + spec.hole_diameters[designer.mid_cut]*0.75
        #cut1 += spec.inner(cut1)*0.5
        #cut2 = spec.hole_positions[-1] + spec.hole_diameters[-1]*0.75
        #cut2 += spec.inner(cut2)*0.5
        #cut2 = max(cut2, true_length*0.5)
        ##cut3 = true_length * 0.7
        #cut3 = cut1 + (true_length-cut1)*0.5 
           
        if not self.mill:
            #self.segment([ cut2 ], spec.length, up=True)
            #self.segment([ cut1, cut2 ], spec.length, up=True)
            #self.segment([ cut0, cut1, cut3 ], spec.length, up=True)
        
            for division in designer.divisions:
                cuts = [ ]
                for hole, above in division:
                    cut = spec.hole_positions[hole] + spec.hole_diameters[hole]*0.75
                    #cut += spec.outer(cut)*0.5
                    cut += (true_length-cut)*above
                    cuts.append(cut)
                    
                cuts.append(spec.length)
                    
                self.segment(cuts, up=True)
        
        else:
            mid1 = spec.hole_positions[2]*0.5+spec.hole_diameters[2]*0.5-spec.hole_diameters[3]*0.5+spec.hole_positions[3]*0.5
            mid2 = spec.hole_positions[5]+spec.hole_diameters[5]
        
            upper_segments, lower_segments = pack.plan_segments([mid1/true_length,mid2/true_length], self.mill_length / true_length)
            
            whistle_jaw = whistle_jaw_clipper.copy()
            whistle_jaw.clip(self.working.outside)
            whistle_jaw.remove(self.working.bore)
            #whistle_jaw_space = whistle_jaw_clipper.copy()
            #whistle_jaw_space.clip(self.working.bore)
            whistle_jaw_space = self.working.bore.copy()
            
            whistle_jaw.rotate(1,0,0, 90)
            whistle_jaw_space.rotate(1,0,0, 90)
            
            offset = -whistle_jaw.extent().zmin
            whistle_jaw.move(0,0,offset)
            whistle_jaw_space.move(0,0,offset)
            
            self.save(whistle_jaw, 'jaw')
            jaw_packable = [ pack.Packable([ whistle_jaw, whistle_jaw_space ], 90, self.mill_diameter) ]

            self.working.bore.add(whistle_jaw_clipper)
            self.working.outside.remove(whistle_jaw_clipper)
                        
            pack.cut_and_pack(
                self.working.outside, self.working.bore,
                upper_segments, lower_segments,
                xsize=self.mill_length, 
                ysize=self.mill_width, 
                zsize=self.mill_thickness,
                bit_diameter=self.mill_diameter,
                save=self.save,
                
                extra_packables=[ jaw_packable ],
            )



if __name__ == '__main__':
    nesoni.run_tool(Make_whistle)





"""

F-shawm
theory  495.5

design  438.4
actual  399
ratio   

"""


import sys, os, math
sys.path.insert(0, os.path.split(__file__)[0])

from . import design, make, profile, shape, pack, make_bauble

from nesoni import config

@config.help("""\
Produce 3D models using the output of "demakein make-shawm:".
""")
@config.Bool_flag('decorate', 'Add some decorative rings.')
@config.Bool_flag('bauble', 'Add a gratuitous bauble to the end.')
class Make_shawm(make.Make_millable_instrument):
    decorate = True
    bauble = False
    
    def run(self):
        spec = self.working.spec
        
        length = spec.length * 0.865 #Reed accounts for part of length, etc
        # 0.875 needed a rather short reed (shapeways tenor, November 2012)
        
        #length = spec.inner.pos[-2]
        #assert spec.inner(length) == self.working.designer.bore
        #self.log.log('Generated shape is %.0f%% of length of simulated instrument, rest is reed.\n' % (length*100.0 / spec.length))        
        #self.log.log('(Effective) reed length: %.1fmm\n' % (spec.length-length))
        
        outer_profile = spec.outer.clipped(0,length)
        
        m = outer_profile.maximum()
        
        if self.bauble:
            end_dock_length = 5.0
            print 'Bauble dock: %.1fmm diameter, %.1fmm length' % (m, end_dock_length)
            bauble = make_bauble.Make_bauble(self.working_dir,dock_length=end_dock_length,dock_diameter=m).run()
        else:
            end_dock_length = m * 0.05
        fixer = profile.make_profile([(end_dock_length,m),(end_dock_length,m),(end_dock_length+m*0.333,0.0)])
        outer_profile = outer_profile.max_with(fixer)
        
        dock_diam = self.working.designer.bore * 4.0
        dock_length = 10.0
        fixer = profile.make_profile([(length-dock_length*1.25,0.0),(length-dock_length,dock_diam)])
        outer_profile = outer_profile.max_with(fixer)
        
        if self.decorate:
            if not self.bauble:
                outer_profile = make.decorate(outer_profile, 0.0, 1.0, 0.05)
            outer_profile = make.decorate(outer_profile, length-dock_length*1.25, -1.0, 0.15)
        
        n_holes = self.working.designer.n_holes
        if n_holes == 6:
            hole_horiz_angles = [ 0.0 ] * 6
            cut2 = spec.inner_hole_positions[0]*0.5+spec.inner_hole_positions[1]*0.5
            cut3 = spec.inner_hole_positions[2]*0.5+spec.inner_hole_positions[3]*0.5
            cut4 = spec.inner_hole_positions[5]*1.3-spec.inner_hole_positions[4]*0.3

            cut1a = spec.hole_positions[0]
            cut1a -= outer_profile(cut1a)*0.5
            cut1b = (cut2+outer_profile(cut2)*0.25) * 0.5
            cut1 = min(cut1a,cut1b)

        else:
            hole_horiz_angles = [ -20.0 ] + [ 0.0 ] * 6 + [ 180.0 ]
            cut1 = spec.hole_positions[0]*0.5+spec.hole_positions[1] * 0.5        

            cut2 = spec.inner_hole_positions[2]*0.5+spec.inner_hole_positions[3] * 0.5
            cut3 = spec.inner_hole_positions[3]*0.5+spec.inner_hole_positions[4] * 0.5
            cut4 = spec.inner_hole_positions[4]*0.5+spec.inner_hole_positions[5] * 0.5
        
        cut5a = spec.hole_positions[-1]
        cut5a += outer_profile(cut5a)*0.5        
        cut5b = length*0.5+cut3*0.5
        cut5b -= outer_profile(cut5b)*0.25
        cut5 = max(cut5a, cut5b)

        if self.mill:
            cut0 = cut1*0.5
            cut6 = length*0.5+cut5*0.5
            upper_segments, lower_segments = pack.plan_segments([cut0/length,cut1/length,cut2/length,cut3/length,cut4/length,cut5/length,cut6/length], self.mill_length / length)
        
        self.make_instrument(
            inner_profile=spec.inner.clipped(-50,length+50),
            outer_profile=outer_profile,
            hole_positions=spec.hole_positions,
            hole_diameters=spec.hole_diameters,
            hole_vert_angles=spec.hole_angles,
            hole_horiz_angles=hole_horiz_angles,
            xpad = [ 0.0 ] * n_holes,
            ypad = [ 0.0 ] * n_holes,
            with_fingerpad = [ True ] * n_holes,
        )
        
        if self.bauble:
            bauble.rotate(1,0,0, 180)
            bauble.move(0,0,end_dock_length)
            binst = self.working.instrument.copy()
            binst.add(bauble)
            self.save(binst, 'baubled-instrument')

        if not self.mill:
            self.segment([ cut1, cut3, cut5 ], up=True)
            self.segment([ cut2, cut4 ], up=True)
            self.segment([ cut3 ], up=True)
        else:
            pack.cut_and_pack(
                self.working.outside, self.working.bore,
                upper_segments, lower_segments,
                xsize=self.mill_length, 
                ysize=self.mill_width, 
                zsize=self.mill_thickness,
                bit_diameter=self.mill_diameter,
                save=self.save,
            )
        
        #self.working.instrument.rotate(0,1,0, 180)
        #self.working.instrument.move(0,0,length)
        #need to flip spec
        #self.segment([ length-cut3, length-cut2, length-cut1 ], length)
        
        
        #shape.Make_instrument.run(self)
        #        
        #designer = self.designer
        #spec = self.instrument
        #
        ##length = spec.length * 0.91 # Shorten to allow reed effective length. TODO: Tune this
        #length = spec.length * 0.875 # Based on trimming alto_shawm
        #width = max(spec.outer.low)
        #
        #instrument = shape.extrude_profile(spec.outer.clipped(0,length))
        #outside = shape.extrude_profile(spec.outer.clipped(0,length))
        #bore = shape.extrude_profile(spec.inner.clipped(-50,length+50))
        #
        #xpad = [ 0.0 ] * 8
        #ypad = [ 0.0 ] * 8
        #angle = [ -20.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 180.0 ]
        #drill_outside = [ 1 ] + [ 0 ] * 7
        #
        #for i, pos in enumerate(spec.hole_positions):
        #    height = spec.outer(pos)*0.5 + 4.0
        #    hole = shape.prism(
        #        height, spec.hole_diameters[i],
        #        shape.squared_circle(xpad[i], ypad[i]).with_effective_diameter
        #    )
        #    hole.rotate(1,0,0, -90)
        #    hole.rotate(0,0,1, angle[i])
        #    hole.move(0,0,pos)
        #    bore.add(hole)
        #    if drill_outside[i]:
        #        outside.remove(hole)
        #
        #instrument.remove(bore)
        #
        #self.save(instrument,'instrument.stl')
        #
        #shapes = pack.cut_and_pack(
        #    outside, bore,
        #    self.SCHEMES[self.scheme][0], self.SCHEMES[self.scheme][1],
        #    xsize=self.SCHEMES[self.scheme][2], ysize=self.SCHEMES[self.scheme][3], zsize=self.thickness,
        #    bit_diameter=3.0,
        #    res=10,
        #    output_dir=self.output_dir
        #)
        #
        #shape.show_only(instrument, *shapes)        
        #
        
        #if self.forms == 1:
        #    lower, upper = shape.make_formwork(
        #        outside, bore, length,
        #        [ 0.0, 0.65, 1.0 ],
        #        [ 0.0, 0.4, 1.0 ],
        #        3.0, 6.0, thickness[0], 200.0
        #    )
        #    
        #    self.save(lower,'lower.stl')
        #    self.save(upper,'upper.stl')
        #    
        #    shape.show_only(instrument, lower, upper)
        #
        #elif self.forms == 2:
        #    lower1, upper1 = shape.make_formwork(
        #        outside, bore, length,
        #        [ 0.0, 0.3, 0.6, 1.0 ],
        #        [ ],
        #        3.0, 6.0, thickness[0], 200.0
        #    )
        #    lower2, upper2 = shape.make_formwork(
        #        outside, bore, length,
        #        [ ],
        #        [ 0.0, 0.4, 0.7, 1.0 ],
        #        3.0, 6.0, thickness[1], 200.0
        #    )
        #    
        #    self.save(lower1,'lower1.stl')
        #    self.save(upper1,'upper1.stl')
        #    self.save(lower2,'lower2.stl')
        #    self.save(upper2,'upper2.stl')
        #    
        #    shape.show_only(instrument, lower1,upper1,lower2,upper2)
        #
        #else:
        #     assert False, 'Unsupported number of forms'
    

if __name__ == '__main__': 
    shape.main_action(Make_shawm())




import math

from nesoni import config

from . import design, optimize

class Working(object): pass

@config.help(
    'Modelling the mouthpiece is more difficult than modelling the body of an '
    'instrument. Some parameters are most easily determined empirically.',
    'This tool tries to explain observed frequencies obtained from an instrument '
    'by tweaking parameters to do with the mouthpiece. '
    'Resultant parameters should then result in a correctly tuned instrument '
    'when the design tool is run again.',
    )
@config.Positional(
    'tweak',
    'Comma separated list of parameters to tweak.'
    )
@config.Main_section(
    'observations',
    'Comma separated lists of frequency followed by '
    'whether each finger hole is open (0) or closed (1) '
    '(from bottom to top).'
    )
class Tune(config.Action_with_working_dir):
    tweak = None
    observations = [ ]
    
    def _constraint_score(self, state):
        #All positive
        return sum( max(-item,0.0) for item in state )
    
    def _errors(self, state):
        mod = self.working.designer(
            **dict(zip(self.working.parameters,state))
            )
        
        instrument = mod.patch_instrument(
            mod.unpack(self.working.designer.state_vec)
            )
        instrument.prepare()
        
        errors = [ ]
        
        s = 1200.0/math.log(2)
        for item in self.observations:
            parts = item.split(',')
            assert len(parts) == (mod.n_holes+1)
            fingers = [ int(item2) for item2 in parts[1:] ]
            w_obtained = design.SPEED_OF_SOUND / float(parts[0])
            w_expected, grad = instrument.true_wavelength_near(w_obtained, fingers, mod.max_grad)

            errors.append( (math.log(w_obtained)-math.log(w_expected))*s )
    
        return errors
    
    def _score(self, state):
        errors = self._errors(state)
        p = 2
        return (sum( abs(item**p) for item in errors ) / max(1,len(errors)))**(1.0/p)
    
    def _report(self, state, etc=[]):        
        for error, observation in zip(self._errors(state),self.observations):
            print '%6.1f cents  %s' % (error, observation)
        
        print
        
        for name, value in zip(self.working.parameters, state):
            print '%s %.3f' % (name, value)
        
        print
    
    def run(self):
        self.working = Working()
        self.working.designer = design.load(self.working_dir)
        self.working.parameters = [ ]
        
        for item in self.tweak.split(','):
            for item2 in self.working.designer.parameters:
                if item2.shell_name().lstrip('-') == item.lstrip('-'):
                    self.working.parameters.append(item2.name)
                    break
            else:
                assert False, 'Unknown parameter: %s' % item
        
        initial = [ 
            getattr(self.working.designer,item)
            for item in self.working.parameters
            ]
        
        state = optimize.improve(
            self.shell_name(), 
            self._constraint_score, 
            self._score, 
            initial,
            monitor=self._report
            )
        
        self._report(state)
        
        
        
        



import math

from . import design, shape, profile, pack

from nesoni import config, legion

def decorate(prof, pos, align, amount=0.2):
    amount = prof(pos)*amount
    pos += amount * align
    deco_profile = profile.Profile(
        [ pos+amount*i for i in [-1,0,1]],
        [ amount*i      for i in [0,1,0] ],
    )    
    return prof + deco_profile.clipped(prof.start(),prof.end())

@config.Float_flag('mill_diameter', 'Diameter of milling bit.')
@config.Bool_flag('mill_ball', 'Is it a ball-mill rather than an end-mill? Extra depth will be added so the bit cuts through into the base.')
class Miller(config.Configurable):
    mill_diameter = 3.0
    mill_ball = True
        
    def miller(self, pattern, xribs=[], yribs=[]):    
        extent = pattern.extent()
        height = extent.zmax - extent.zmin
        
        bit_pad = self.mill_diameter * 1.25
        extra_depth = self.mill_diameter*0.5 if self.mill_ball else 0.0
        a = bit_pad
        b = bit_pad + (height+extra_depth)/10.0
        pad_cone = shape.extrude_profile(profile.Profile(
                [extent.zmin-extra_depth,extent.zmax],[a*2,b*2]
            ), 
            cross_section=lambda d: shape.circle(d,16))
        
        extra = b + 0.5
        
        extent = pattern.extent()
        mill = shape.block(
            extent.xmin-extra,       extent.xmax+extra,
            extent.ymin-extra,       extent.ymax+extra,
            extent.zmin-extra_depth, extent.zmax,
        )
        
        mill.remove( pattern.polygon_mask().to_3().minkowski_sum(pad_cone) )
        mill.add( pattern )
        
        rib_width = 1.0
        rib_height = 2.0
        for x in xribs:
            block = shape.block(
                x-rib_width*0.5, x+rib_width*0.5,
                extent.ymin-extra,extent.ymax+extra,
                extent.zmin-extra_depth,extent.zmin+rib_height
                )
            mill.add(block)
        
        for y in yribs:
            block = shape.block(
                extent.xmin-extra,extent.xmax+extra,
                y-rib_width*0.5, y+rib_width*0.5,
                extent.zmin-extra_depth,extent.zmin+rib_height
                )
            mill.add(block)

        return mill


@config.Bool_flag('draft', 'Draft output')
class Make_base(config.Action):
    draft = False
    
    # We'll need a lot of memory, so run exclusively
    def cores_required(self):
        return legion.coordinator().get_cores()

    def save(self, shape, prefix):
        prefix = self.get_workspace()/prefix
        
        shape.save(prefix + '.stl')
        
        import sketch
        sketch.sketch(shape, prefix + '-sketch.svg')

    def _before_run(self):
        if self.draft:
            shape.draft_mode()
        self.get_workspace()

class Make(config.Action_with_output_dir, Make_base):
    pass


class Working(object): pass

@config.String_flag('prefix', 'Output file prefix (useful if you want to make your design several different ways).')
@config.Bool_flag('fingerpads',
    'Flatten area around finger holes, '
    'possibly producing a better seal.'
    )
@config.String_flag('join',
    'Printing: Join type, options are:\n'
    'weld - ABS pieces to be joined by acetone welding, or similar.\n'
    'straight - Straight socket.\n'
    'taper - Tapered socket.\n'
    )
@config.Float_flag('gap', 
    'Printing: Amount of gap all around in the joins between segments '
    'for straight or tapered joins. '
    'The best value for this will depend on the accuracy of your printer. '
    'If a joint is too loose and leaks, it can be sealed using wax or similar.')
@config.Bool_flag('thick_sockets',
    'Add some extra thickness around sockets.'
    )
class Make_instrument(config.Action_with_working_dir, Make_base):
    gap = 0.2
    fingerpads = False
    join = 'weld'
    thick_sockets = False
    prefix = ''

    def _before_run(self):
        self.working = Working()
        self.working.designer = design.load(self.working_dir)
        self.working.spec = self.working.designer.instrument

    def _after_run(self):
        del self.working
        
    def save(self, shape, prefix):
        Make_base.save(self, shape, self.prefix+prefix)

    def get_cuts(self):
        length = self.working.top
        spec = self.working.spec
    
        result = [ ]
        for division in self.working.designer.divisions:
            cuts = [ ]
            for hole, above in division:
                if hole >= 0:
                    cut = spec.hole_positions[hole] + 2*spec.hole_diameters[hole]
                else:
                    cut = 0.0
                cut += (length-cut)*above
                cuts.append(cut)
            result.append(cuts)
        return result

    def make_segments(self, up):
        for item in self.get_cuts():
            self.segment(item, up)

    def make_instrument(
             self,
             inner_profile, outer_profile, 
             hole_positions, hole_diameters, hole_vert_angles, hole_horiz_angles,
             xpad, ypad, with_fingerpad,
             outside_extras = [], bore_extras = []):
        outside = shape.extrude_profile(outer_profile)
        instrument = outside.copy()        
        bore = shape.extrude_profile(inner_profile)
                      
        for i, pos in enumerate(hole_positions):
            angle = hole_vert_angles[i]
            radians = angle*math.pi/180.0
        
            height = outer_profile(pos)*0.5 
            inside_height = inner_profile(pos)*0.5 
            shift = math.sin(radians) * height
            
            #hole_length = (
            #    math.sqrt(height*height+shift*shift) + 
            #    hole_diameters[i]*0.5*abs(math.sin(radians)) + 
            #    4.0)
            
            hole_diameter_correction = math.cos(radians) ** -0.5
            hole_diameter = hole_diameters[i] * hole_diameter_correction
                
            cross_section = shape.squared_circle(xpad[i], ypad[i]).with_effective_diameter
            
            #h1 = inside_height*0.75
            h1 = inside_height*0.5
            shift1 = math.sin(radians)*h1
            h2 = height*1.5
            shift2 = math.sin(radians)*h2
            hole = shape.extrusion([h1,h2],
                [ cross_section(hole_diameter).offset(0.0,shift1),
                  cross_section(hole_diameter).offset(0.0,shift2) ])
            hole.rotate(1,0,0, -90)
                
            #hole = shape.prism(
            #    hole_length, hole_diameters[i],
            #    cross_section)                
            #hole.rotate(1,0,0, -90-angle)
            
            hole.rotate(0,0,1, hole_horiz_angles[i])
            hole.move(0,0,pos + shift)
            
            if with_fingerpad[i] and self.fingerpads:
                pad_height = height*0.5+0.5*math.sqrt(height**2-(hole_diameters[i]*0.5)**2)
                pad_depth = (pad_height-inside_height)
                pad_mid = pad_depth*0.25
                pad_diam = hole_diameter*1.3
                fingerpad = shape.extrude_profile(
                    profile.Profile([-pad_depth,-pad_mid,0.0],[pad_diam+pad_mid*2,pad_diam+pad_mid*2,pad_diam]),
                    cross_section=cross_section)
                fingerpad_negative = shape.extrude_profile(
                    profile.Profile([0.0,pad_mid,pad_depth],[pad_diam,pad_diam+pad_mid*8,pad_diam+pad_mid*8]),
                    cross_section=cross_section)
                
                wall_angle = -math.atan2(0.5*(outer_profile(pos+pad_diam*0.5)-outer_profile(pos-pad_diam*0.5)),pad_diam)*180.0/math.pi
                fingerpad.rotate(1,0,0, wall_angle)
                fingerpad_negative.rotate(1,0,0, wall_angle)
                
                fingerpad.move(0,0,pad_height)
                fingerpad_negative.move(0,0,pad_height)
                fingerpad.rotate(1,0,0, -90)
                fingerpad_negative.rotate(1,0,0, -90)
                fingerpad.rotate(0,0,1, hole_horiz_angles[i])
                fingerpad_negative.rotate(0,0,1, hole_horiz_angles[i])
                fingerpad.move(0,0,pos) # - 0.2*hole_diameters[i]*math.sin(radians)) #????????????????????
                fingerpad_negative.move(0,0,pos) # - 0.2*hole_diameters[i]*math.sin(radians)) #?????????????????????
                outside.add(fingerpad)
                outside.remove(fingerpad_negative)
                instrument.add(fingerpad)
                instrument.remove(fingerpad_negative)

            bore.add(hole)
            if angle or hole_horiz_angles[i]:
                outside.remove(hole)
        
        for item in outside_extras:
            outside.add(item)
            instrument.add(item)
        for item in bore_extras:
            bore.add(item)
        
        instrument.remove(bore)
        instrument.rotate(0,0,1, 180)
        
        self.working.instrument = instrument
        self.working.outside = outside
        self.working.bore = bore
        self.working.top = instrument.size()[2]        
        self.save(instrument, 'instrument')

    def segment(self, cuts, up=False):
        working = self.working
        length = working.top
        remainder = working.instrument.copy()
        if self.thick_sockets:
            bore = self.working.bore
        inner = working.spec.inner
        outer = working.spec.outer
        
        if up:
            cuts = [ length-item for item in cuts[::-1] ]
            remainder.rotate(0,1,0, 180)
            remainder.move(0,0,length)
            if self.thick_sockets:
                bore = bore.copy()
                bore.rotate(0,1,0, 180)
                bore.move(0,0,length)
                
            inner = inner.reversed().moved(length)
            outer = outer.reversed().moved(length)
        
        if self.join == 'weld':
            socket = self._weld_join
        elif self.join == 'straight':
            socket = self._straight_socket 
        elif self.join == 'taper':
            socket = self._tapered_socket
        else:
            assert False, 'Unknown join type: '+self.join
        
        shapes = [ ]
        for cut in cuts:
            d1 = inner(cut)
            d4 = outer(cut)
            d5 = outer.maximum() * 2.0
            #p1 = cut-d4*0.4
            #p3 = cut+d4*0.4
            
            sock_length = d4*0.8
            p1 = cut-sock_length
            p3 = cut
            if not up and self.join != 'weld':
                p1 += sock_length
                p3 += sock_length
            
            if self.thick_sockets:
                d4_orig = d4
                d4 += min(d4*0.2, (d4-d1)*0.5)
                prof_thicker = profile.Profile(
                    [ p1-(d4-d4_orig), p1, p3 ],
                    [ (d1+d4)*0.5, d4, d4 ],
                )
                thicker = shape.extrude_profile(prof_thicker)
                thicker.remove(bore)
                remainder.add(thicker)
                del thicker
            
            mask_inside, mask_outside = socket(
                p1,
                p3,
                length,
                d1,
                d4,
                d5
                )
            
            item = remainder.copy()
            item.remove(mask_outside)                
            remainder.clip(mask_inside)
            shapes.append(item)
            del mask_inside
            del mask_outside
        shapes.append(remainder)
        shapes = shapes[::-1]
        
        for i, item in enumerate(shapes):
            item.rotate(0,1,0, 180)
            item.position_nicely()
            self.save(item, '%d-piece-%d' % (len(shapes),i+1))
        
        #size = outer.maximum() * 4.0
        #template = pack.Pack(size,size,5000.0)
        #packables = [ [pack.Packable([item], 0.0, 1.0)] for item in shapes ]
        #[ packed ] = pack.pack(template, packables)
        #render = packed.render_print()
        #render.position_nicely()
        #self.save(render, '%d-piece-all' % len(shapes))
        
        #self.working.segments = shapes

    def _straight_socket(
                self,
                p1,
                p3,
                length,
                d1,
                d3,
                d4
                ):       
            d2 = (d1+d3) / 2.0
            
            p2 = p1 + (d2-d1)*0.5
                    
            d1a = d1 - self.gap
            p1b = p1 - self.gap
            
            d2a = d2 - self.gap
            d2b = d2 + self.gap
            
            prof_inside = profile.Profile(
                [ p1,  p2,  p3,  length+50 ],
                [ d1a, d2a, d2a, d4 ],
                [ d1a, d2a, d4,  d4 ],
            )
            prof_outside = profile.Profile(
                [ p1b, p2,  p3,  length+50 ],
                [ d1,  d2b, d2b, d4 ],
                [ d1,  d2b, d4,  d4 ],
            )
            mask_inside = shape.extrude_profile(prof_inside)
            mask_outside = shape.extrude_profile(prof_outside)
            return mask_inside, mask_outside

    def _tapered_socket(
                self,
                p1,
                p3,
                length,
                d1,
                d4,
                d5
                ):       
            d3 = (d1+d4) / 2.0
            d2 = (d1+d3) / 2.0
            
            p2 = p1 + (d2-d1)
                    
            d1a = d1 - self.gap
            p1b = p1 - self.gap
            
            d2a = d2 - self.gap
            d2b = d2 + self.gap

            d3a = d3 - self.gap
            d3b = d3 + self.gap
            
            prof_inside = profile.Profile(
                [ p1,  p2,  p3,  length+50 ],
                [ d1a, d2a, d3a, d5 ],
                [ d1a, d2a, d5,  d5 ],
            )
            prof_outside = profile.Profile(
                [ p1b, p2,  p3,  length+50 ],
                [ d1,  d2b, d3b, d5 ],
                [ d1,  d2b, d5,  d5 ],
            )
            mask_inside = shape.extrude_profile(prof_inside)
            mask_outside = shape.extrude_profile(prof_outside)
            return mask_inside, mask_outside

    def _weld_join(self,z0,z1,zmax,d0,d1,dmax):
        prof = profile.Profile(
            [ z1, zmax+50 ],
            [ dmax, dmax]
            )
        mask_upper = shape.extrude_profile(prof)
        mask_lower = mask_upper.copy()
        
        triangle = shape.Loop([
            (0.5, 0.0),
            (0.0, math.sqrt(0.75)),
            (-0.5, 0.0),
            ])
        triangle_upper = triangle.scale(d0*0.5+self.gap)
        triangle_lower  = triangle.scale(d0*0.5-self.gap)
        d1_3 = d0*0.6666+d1*0.3334
        d2_3 = d0*0.3334+d1*0.6666
        for i in xrange(1,5):
            bump = shape.extrusion(
                [ d1_3*0.5-self.gap*0.5, d2_3*0.5-self.gap*0.5, dmax*0.5 ],
                [ triangle_upper.scale(0.0), triangle_upper, triangle_upper ]
                )
            bump.rotate(1,0,0,-90)
            bump.rotate(0,0,1,180+360.0/5*i)
            bump.move(0,0,z1)
            mask_upper.add(bump)

            bump = shape.extrusion(
                [ d1_3*0.5+self.gap*0.5, d2_3*0.5+self.gap*0.5, dmax*0.5 ],
                [ triangle_lower.scale(0.0), triangle_lower, triangle_lower ]
                )
            bump.rotate(1,0,0,-90)
            bump.rotate(0,0,1,180+360.0/5*i)
            bump.move(0,0,z1)
            mask_lower.add(bump)
        
        return mask_lower, mask_upper

        
@config.Bool_flag('mill', 'Create shapes for milling (rather than 3D printing).')
@config.Float_flag('mill_diameter', 'Milling: Bit diameter for milling (affects gap size when packing pieces).')
@config.Float_flag('mill_length', 'Milling: Wood length for milling.')
@config.Float_flag('mill_width', 'Milling: Wood width for milling.')
@config.Float_flag('mill_thickness', 'Milling: Wood thickness for milling.')
class Make_millable_instrument(Make_instrument):
    mill = False
    mill_diameter = 3.0    
    mill_length = 180.0
    mill_width = 130.0
    mill_thickness = 19.0
    
    def make_parts(self, up):
        if self.mill:
            self.make_workpieces()
        else:
            self.make_segments(up)
    
    def make_workpieces(self):
        length = self.working.top
        cuts = sorted(set( item/length for scheme in self.get_cuts() for item in scheme ))
        upper_segments, lower_segments = pack.plan_segments(cuts, self.mill_length / length)
        pack.cut_and_pack(
            self.working.outside, self.working.bore,
            upper_segments, lower_segments,
            xsize=self.mill_length, 
            ysize=self.mill_width, 
            zsize=self.mill_thickness,
            bit_diameter=self.mill_diameter,
            save=self.save,
        )
        





# Synthesize a procedure to numerically integrate the 3rd order poly spiral

tex = False

if tex:
    mulsym = ' '
else:
    mulsym = ' * '

class Poly:
    def __init__(self, p0, coeffs):
        self.p0 = p0
        self.coeffs = coeffs
    def eval(self, x):
        y = x ** self.p0
        z = 0
        for c in coeffs:
            z += y * c
            y *= x
        return z

def add(poly0, poly1, nmax):
    lp0 = len(poly0.coeffs)
    lp1 = len(poly1.coeffs)
    p0 = min(poly0.p0, poly1.p0)
    n = min(max(poly0.p0 + lp0, poly1.p1 + lp1), nmax) - p0
    if n <= 0: return Poly(0, [])
    coeffs = []
    for i in range(n):
        c = 0
        if i >= poly0.p0 - p0 and i < lp0 + poly0.p0 - p0:
            c += poly0.coeffs[i + p0 - poly0.p0]
        if i >= poly1.p0 - p0 and i < lp1 + poly1.p0 - p0:
            c += poly1.coeffs[i + p0 - poly1.p0]
        coeffs.append(c)
    return Poly(p0, coeffs)

def pr(str):
    if tex:
        print str, '\\\\'
    else:
        print '\t' + str + ';'

def prd(str):
    if tex:
        print str, '\\\\'
    else:
        print '\tdouble ' + str + ';'

def polymul(p0, p1, degree, basename, suppress_odd = False):
    result = []
    for i in range(min(degree, len(p0) + len(p1) - 1)):
        terms = []
        for j in range(i + 1):
            if j < len(p0) and i - j < len(p1):
                t0 = p0[j]
                t1 = p1[i - j]
                if t0 != None and t1 != None:
                    terms.append(t0 + mulsym + t1)
        if terms == []:
            result.append(None)
        else:
            var = basename % i
            if (j % 2 == 0) or not suppress_odd:
                prd(var + ' = ' + ' + '.join(terms))
            result.append(var)
    return result

def polysquare(p0, degree, basename):
    result = []
    for i in range(min(degree, 2 * len(p0) - 1)):
        terms = []
        for j in range((i + 1)/ 2):
            if i - j < len(p0):
                t0 = p0[j]
                t1 = p0[i - j]
                if t0 != None and t1 != None:
                    terms.append(t0 + mulsym + t1)
        if len(terms) >= 1:
            if tex and len(terms) == 1:
                terms = ['2 ' + terms[0]]
            else:
                terms = ['2' + mulsym + '(' + ' + '.join(terms) + ')']
        if (i % 2) == 0:
            t = p0[i / 2]
            if t != None:
                if tex:
                    terms.append(t + '^2')
                else:
                    terms.append(t + mulsym + t)
        if terms == []:
            result.append(None)
        else:
            var = basename % i
            prd(var + ' = ' + ' + '.join(terms))
            result.append(var)
    return result

def mkspiro(degree):
    if tex:
        us = ['u = 1']
        vs = ['v =']
    else:
        us = ['u = 1']
        vs = ['v = 0']
    if tex:
        tp = [None, 't_{11}', 't_{12}', 't_{13}', 't_{14}']
    else:
        tp = [None, 't1_1', 't1_2', 't1_3', 't1_4']
    if tex:
        prd(tp[1] + ' = k_0')
        prd(tp[2] + ' = \\frac{k_1}{2}')
        prd(tp[3] + ' = \\frac{k_2}{6}')
        prd(tp[4] + ' = \\frac{k_3}{24}')
    else:
        prd(tp[1] + ' = km0')
        prd(tp[2] + ' = .5 * km1')
        prd(tp[3] + ' = (1./6) * km2')
        prd(tp[4] + ' = (1./24) * km3')
    tlast = tp
    coef = 1.
    for i in range(1, degree - 1):
        tmp = []
        tcoef = coef
        #print tlast
        for j in range(len(tlast)):
            c = tcoef / (j + 1)
            if (j % 2) == 0 and tlast[j] != None:
                if tex:
                    tmp.append('\\frac{%s}{%.0f}' % (tlast[j], 1./c))
                else:
                    if c < 1e-9:
                        cstr = '%.16e' % c
                    else:
                        cstr = '(1./%d)' % int(.5 + (1./c))
                    tmp.append(cstr + ' * ' + tlast[j])
            tcoef *= .5
        if tmp != []:
            sign = ('+', '-')[(i / 2) % 2]
            var = ('u', 'v')[i % 2]
            if tex:
                if i == 1: pref = ''
                else: pref = sign + ' '
                str = pref + (' ' + sign + ' ').join(tmp)
            else:
                str = var + ' ' + sign + '= ' + ' + '.join(tmp)
            if var == 'u': us.append(str)
            else: vs.append(str)
        if i < degree - 1:
            if tex:
                basename = 't_{%d%%d}' % (i + 1)
            else:
                basename = 't%d_%%d' % (i + 1)
            if i == 1:
                tnext = polysquare(tp, degree - 1, basename)
                t2 = tnext
            elif i == 3:
                tnext = polysquare(t2l, degree - 1, basename)
            elif (i % 2) == 0:
                tnext = polymul(tlast, tp, degree - 1, basename, True)
            else:
                tnext = polymul(t2l, t2, degree - 1, basename)
            t2l = tlast
            tlast = tnext
        coef /= (i + 1)
    if tex:
        pr(' '.join(us))
        pr(' '.join(vs))
    else:
        for u in us:
            pr(u)
        for v in vs:
            pr(v)

if __name__ == '__main__':
    mkspiro(12)



"""

A basic reedpipe.

Note that the finger holes are underconstrained
by the fingering system.
Constraints such as balance and min_hole_spacing
can be added to produce a comfortable instrument.

Note also the reed has some virtual length, 
so the real instrument length will be shorter than designed.

"""

import demakein, nesoni

class Reedpipe(demakein.design.Instrument_designer):
    closed_top = True

    inner_diameters = [ 6.0, 6.0 ]
    outer_diameters = [ 10.0, 10.0 ]
    
    min_hole_diameters = [ 3.0 ]*7
    max_hole_diameters = [ 5.0 ]*7
    
    #min_hole_spacing = [ 15.0 ]*6
    #and/or
    balance = [ 0.07 ]*5
    
    initial_length = demakein.design.wavelength('C4') * 0.25
    
    fingerings = [
        ('C4',  [1,1,1,1,1,1,1]),
        ('D4',  [0,1,1,1,1,1,1]),
        ('E4',  [0,0,1,1,1,1,1]),
        ('F4',  [0,0,0,1,1,1,1]),
        ('G4',  [0,0,0,0,1,1,1]),
        ('A4',  [0,0,0,0,0,1,1]),
        ('B4',  [0,0,0,0,0,0,1]),
        ('C5',  [0,0,0,0,0,1,0]),
        ]    
    
if __name__ == '__main__': 
    nesoni.run_tool(Reedpipe)




"""

Note:

The "reed" has some virtual length.

Drinking straw needs to be 0.85 of simulated length.

"""

import copy

import demakein, nesoni

class Reedpipe(demakein.design.Instrument_designer):
    def patch_instrument(self, inst):
        inst = copy.copy(inst)
        inst.length /= 0.85
        return inst


class Diatonic(Reedpipe):
    closed_top = True

    inner_diameters = [ 6.0, 6.0 ]
    outer_diameters = [ 6.0, 6.0 ]
    
    min_hole_diameters = [ 4.0 ]*7
    max_hole_diameters = [ 4.0 ]*7
    
    initial_length = demakein.design.wavelength('C4') * 0.25 * 0.85
    fingerings = [
        ('C4',  [1,1,1,1,1,1,1]),
        ('D4',  [0,1,1,1,1,1,1]),
        ('E4',  [0,0,1,1,1,1,1]),
        ('F4',  [0,0,0,1,1,1,1]),
        ('G4',  [0,0,0,0,1,1,1]),
        ('A4',  [0,0,0,0,0,1,1]),
        ('B4',  [0,0,0,0,0,0,1]),
        ('C5',  [0,0,0,0,0,1,0]),
    ]    

    
class Pentatonic(Reedpipe):
    closed_top = True

    inner_diameters = [ 6.0, 6.0 ]
    outer_diameters = [ 6.0, 6.0 ]
    
    min_hole_diameters = [ 4.0 ]*5
    max_hole_diameters = [ 4.0 ]*5
    
    initial_length = demakein.design.wavelength('C4') * 0.25 * 0.85
    fingerings = [
        ('C4',  [1,1,1,1,1]),
        ('D4',  [0,1,1,1,1]),
        ('E4',  [0,0,1,1,1]),
        ('G4',  [0,0,0,1,1]),
        ('A4',  [0,0,0,0,1]),
        ('C5',  [0,0,0,0,0]),
    ]    
    

if __name__ == '__main__': 
    nesoni.run_toolbox([ Diatonic, Pentatonic ])






from nesoni import config

from . import shape, profile, make

@config.help(
'Make a cork.',
"""\
The cork will taper from a diameter of diameter - taper-in to a diamter of diameter + taper-out.
""")
@config.Float_flag('length')
@config.Float_flag('diameter')
@config.Float_flag('taper_in')
@config.Float_flag('taper_out')
class Make_cork(make.Make, make.Miller):
    diameter = 10.0
    length = 10.0
    taper_in = 0.25
    taper_out = 0.125

    def run(self):
        d1 = self.diameter + self.taper_out
        d2 = self.diameter - self.taper_in
        l = self.length        
        cork = shape.extrude_profile(profile.make_profile([(0.0,d1),(l,d2)]))
        
        self.save(cork, 'cork')
        
        mill = self.miller(cork)
        self.save(mill, 'mill-cork')



"""

Gwarggllykiiiiillllllmeeeeeeeeecougcougcough

Note: pypy gc does not count c++ allocated memory, triggers major gc infrequently

"""

import os, subprocess, hashlib, gc, time, platform

def update_file(filename, content):
    good = os.path.exists(filename)
    if good:
        with open(filename,'rb') as f:
            current = f.read()
        good = (current == content)
    if not good:
        with open(filename,'wb') as f:
            f.write(content)
    return not good

def execute(command):
    assert 0 == os.system(command)

CMAKE_TEMPLATE = r"""
cmake_minimum_required (VERSION 2.6)

project (Project)

%(cmake_code)s

%(cmake_lib)s
"""

UNBOXED_TYPES = [
    'bool','int','long','unsigned int','unsigned long','double','void','char *','char const*'
]

class Box(object):
    def __init__(self, module, is_reference, cpp_type, value, deleter=None):
        self.module = module
        self.is_reference = is_reference
        self.cpp_type = cpp_type
        self.value = value
        self.deleter = deleter
    
    def __del__(self):
        if self.deleter:
            self.deleter(self.value)
    
    def __repr__(self):    
        return 'Box(%s,%s)' % (self.cpp_type + (' &' if self.is_reference else ''), self.value)
    
    def __str__(self):
        ss = self.module.new('std::stringstream()')
        self.module.do('a<<b', a=ss,b=self)
        cpp_str = self.module('a.str()',a=ss)
        c_str = self.module('a.c_str()',a=cpp_str)
        string = self.module.ffi.string(c_str)
        return string


class Lazy_init(object):
    def __init__(self, *args, **kwargs):
        self._init_args = (args, kwargs)
        self._init_done = False
    
    def __getattr__(self, name):
        if not self._init_done:
            self._lazy_init(*self._init_args[0],**self._init_args[1])
            del self._init_args
            self._init_done = True
        return object.__getattribute__(self, name)

class Module(Lazy_init):
    def _lazy_init(self, build_dir, preamble, cmake_code=''):
        import cffi
        
        self.build_dir = build_dir
        self.preamble = preamble
        self.code = [ ]
        self.cmake_code = cmake_code
        
        self.iteration = 0
        
        self.functions = { }

        self.modules = [ ]
        self.symbol_index = { }

        self.ffi = cffi.FFI()
        self.cdata_type = type(self.ffi.cast('int',0))
        
        self.gc_clock = 0
        
        self.parse_exports(self.preamble)
    
        try:
            with open(self.build_dir + '/hint','rb') as f:
                hint = f.read()
            hint = eval(hint)
            assert hint['preamble'] == self.preamble
            self.hint = hint['hint']
        except Exception:
            self.hint = [ ]
            
    def _gc_tick(self):
        t1 = time.time()
        if t1 >= self.gc_clock:
            gc.collect()
            t2 = time.time()
            self.gc_clock = t1+(t2-t1)*20.0
        
    def get_symbol(self, symbol):
        if symbol not in self.symbol_index:
            for module in self.modules:
                if hasattr(module,symbol):
                    self.symbol_index[symbol] = getattr(module, symbol)
                    break
        return self.symbol_index[symbol]
    
    def parse_exports(self, chunk):
        for line in chunk.split('\n'):
            if '/*export*/' in line:
                self.ffi.cdef(line.strip() + ';')
    
    def load_code(self, chunks):
        if not chunks:
            return
            
        iteration = self.iteration
        self.iteration += 1

        if not os.path.exists(self.build_dir):
            os.mkdir(self.build_dir)

        for chunk in chunks:
            self.parse_exports(chunk)

        filenames = [ ]
        for chunk in chunks:
            code = [ ]
            for line in (self.preamble+'\n'+chunk).split('\n'):
                if '/*export*/' in line:
                    code.append('extern "C" '+line.strip()+'\n')
                else:
                    code.append(line+'\n')            
            code = ''.join(code)
            filename = 'chunk_' + hashlib.sha1(code).hexdigest() + '.cpp'
            filenames.append(filename)
            update_file(self.build_dir+'/'+filename, code)
        
        if platform.system() == 'Darwin':
            cmake_lib = 'add_library (code%d SHARED %s)\n' % (iteration, ' '.join(filenames))
            cmake_code = self.cmake_code
            update_file(self.build_dir+'/CMakeLists.txt', CMAKE_TEMPLATE % locals())
            execute('cd '+self.build_dir+' && cmake .')
            execute('cd '+self.build_dir+' && make')
            self.modules.append( self.ffi.dlopen(self.build_dir+'/libcode%d.dylib' % iteration) )
                
        else:
            cmake_lib = 'add_library (code SHARED %s)\n' % ' '.join(filenames)
            cmake_code = self.cmake_code        
            update_file(self.build_dir+'/CMakeLists.txt', CMAKE_TEMPLATE % locals())
            execute('cd '+self.build_dir+' && cmake .')
            execute('cd '+self.build_dir+' && make && cp libcode.so libcode%d.so' % iteration)
            self.modules.append( self.ffi.dlopen(self.build_dir+'/libcode%d.so' % iteration) )
            execute('cd '+self.build_dir+' && rm libcode%d.so' % iteration)
        
        self.code.extend(chunks)

        with open(self.build_dir + '/hint','wb') as f:
            f.write( repr({'preamble':self.preamble, 'hint':self.code}) )

    def cpp_type(self, item):
        if isinstance(item, self.cdata_type):
            return self.ffi.getctype(self.ffi.typeof(value))
        elif isinstance(item, bool):
            return 'bool'
        elif isinstance(item, int):
            return 'int'
        elif isinstance(item, float):
            return 'double'
        elif isinstance(item, str):
            return 'char const*'
        elif isinstance(item, Box):
            return item.cpp_type
        assert False, 'Unhandled type'
    
    def c_type(self, item):
        if isinstance(item, Box):
            return 'void *'        
        cpp_type = self.cpp_type(item)
        if cpp_type == 'bool':
            return 'int'
        return cpp_type
    
    def c_value(self, c_type, item):
        if isinstance(item, str):
            item = self.ffi.new('char[]', item)
    
        if isinstance(item, Box):
            return self.ffi.cast(c_type, item.value)
        else:
            return self.ffi.cast(c_type, item)
        
    def is_reference(self, item):
        return isinstance(item,Box) and item.is_reference
    
    def require(self, *chunks):
        chunks = list(chunks) + self.hint
        self.hint = [ ]
        
        new_chunks = [ ]
        for chunk in chunks:
            if chunk not in self.code and chunk not in new_chunks:
                new_chunks.append(chunk)

        self.load_code(new_chunks)

    def eval(self, expression, _return_value=True, _as_reference=False, **args):
        """ _as_reference=True : 
                requires return type to be a pointer,
                result will behave as a reference, 
                will be garbage collected
        """
        self._gc_tick()
        
        arg_names = tuple(sorted(args))
        arg_is_reference = tuple( self.is_reference(args[item]) for item in arg_names )
        arg_cpp_types = tuple( self.cpp_type(args[item]) for item in arg_names )
        arg_c_types = [ self.c_type(args[item]) for item in arg_names ]
        arg_c_values = [ self.c_value(c_type, args[item]) for c_type, item in zip(arg_c_types, arg_names) ]
        signature = (_return_value, expression, arg_names, arg_cpp_types, arg_is_reference)
        
        if signature not in self.functions:
            parameters = [ ]
            header = ''
            for i in xrange(len(arg_names)):
                param = arg_names[i]
                c_type = arg_c_types[i]
                cpp_type = arg_cpp_types[i]
                is_reference = arg_is_reference[i]
                if cpp_type != c_type:
                    parameters.append( '%(c_type)s _%(param)s' % locals() )
                    if is_reference:
                        header += '  %(cpp_type)s &%(param)s = *(%(cpp_type)s*)_%(param)s;\n' % locals()
                    else:
                        header += '  %(cpp_type)s %(param)s = (%(cpp_type)s)_%(param)s;\n' % locals()
                else:
                    parameters.append( '%(c_type)s %(param)s' % locals() )
            all_parameters = ','.join(parameters)
            
            name = 'lambda_'+hashlib.sha1(repr(signature)).hexdigest()
            
            if not _return_value:
                as_reference = False
                return_cpp_type = 'void'
                return_c_type = 'void'
                delete_name = None
                code = (
                    '%(return_c_type)s %(name)s(%(all_parameters)s) /*export*/\n' % locals() +
                    '{\n' +
                    header +
                    '  %(expression)s;\n' % locals() +
                    '}\n\n'
                )            
                self.require(code)
                
            else:
                typegetter_name = 'type_'+name
                
                code = (
                    'const char* %(typegetter_name)s(%(all_parameters)s) /*export*/\n' % locals() +
                    '{\n' +
                    header +
                    '  return typeid(( %(expression)s )).name();\n' % locals() +
                    '}\n\n'
                )
                #                    ^ Double brackets so doesn't return function type
                self.require(code)
                
                mangle = self.ffi.string(self.get_symbol(typegetter_name)(*arg_c_values))
                with subprocess.Popen(['c++filt','-t',mangle],stdout=subprocess.PIPE).stdout as f:
                    return_cpp_type = f.read().strip()
                
                if _as_reference:
                    as_reference = True
                    assert return_cpp_type.endswith('*')
                    return_cpp_type = return_cpp_type[:-1]
                    return_c_type = 'void *'
                
                else:
                    as_reference = not (return_cpp_type in UNBOXED_TYPES)
                    if as_reference:
                        expression = 'new ('+return_cpp_type+')(('+expression+'))'
                        return_c_type = 'void *'
                    else:
                        return_c_type = return_cpp_type
                        if return_c_type == 'bool':
                            return_c_type = 'int'
                
                code = (
                    '%(return_c_type)s %(name)s(%(all_parameters)s) /*export*/\n' % locals() +
                    '{\n' +
                    header +
                    '  return (%(return_c_type)s)( %(expression)s );\n' % locals() +
                    '}\n\n'
                )            
                if not as_reference:
                    self.require(code)
                    delete_name = None
                else:
                    delete_name = 'delete_' + hashlib.sha1(return_cpp_type).hexdigest()
                    delete_code = (
                        'void %(delete_name)s(void *item) /*export*/\n'
                        '{\n'
                        '  delete (%(return_cpp_type)s *)item;\n'
                        '}\n\n'
                    ) % locals()
                    
                    self.require(code, delete_code)

            
            self.functions[signature] = (name, as_reference, return_c_type, return_cpp_type, delete_name)
        
        (name, as_reference, return_c_type, return_cpp_type, delete_name) = self.functions[signature]

        if delete_name:
            deleter = self.get_symbol(delete_name)
        else:
            deleter = None

        result = self.get_symbol(name)(*arg_c_values)
        
        if as_reference:
            result = Box(self, as_reference, return_cpp_type, result, deleter)
            
        elif return_cpp_type == 'bool':
            result = bool(result)

        return result
    
    __call__ = eval
    
    def do(self, expression, **args):
        self.eval(expression, _return_value=False, **args)
    
    def as_reference(self, expression, **args):
        """ Treat the result (a pointer) as a reference.
            Result will be garbage collected.
        """
        return self.eval(expression,_as_reference=True,**args)
    
    def new(self, initializer, **args):
        return self.as_reference('new '+initializer,**args)
    
    def call(self, func, *args):
        names = [ '_arg%d' % i for i in xrange(len(args)) ]
        param = dict(zip(names,args))
        expression = func + '('+','.join(names)+')'
        return self.eval(expression, **param)
    
    def iterate(self, expr_begin, expr_end, **args):
        i = self(expr_begin,**args)
        end = self(expr_end,**args)
        while not self('a==b',a=i,b=end):
            yield self('*a',a=i)
            self.do('++a',a=i)
    
    def circulate(self, expression, **args):
        start = self(expression,**args)
        i = self('a',a=start)
        while True:
            yield self('*a',a=i)
            self.do('++a',a=i)
            if self('a==b',a=i,b=start): 
                break
        


if __name__ == '__main__':
    M = Module('__testcpp__', """
#include <typeinfo>
#include <iostream>
#include <vector>
""")

    # Execute a statement
    M.do('std::cout << "Hello world." << std::endl')
    
    # Create a new object (will be deleted by garbage collector)
    vec = M.new('std::vector<int>')
    
    for i in xrange(10):
        M.do('a.push_back(b)', a=vec, b=i)
    
    # for(i=<expr1>;i!=<expr2>;i++) yield *i
    for i in M.iterate('a.begin()','a.end()',a=vec):
        #Compute a value
        square = M('a*a',a=i)
        
        M.do('std::cout << a << std::endl',a=square)




#!/usr/bin/env python3

import copy, math

import profile, design

from nesoni import config

fingerings = {
    'pflute' : [
        ('D4',   [1,1,1,1,1,1]),
        ('E4',   [0,1,1,1,1,1]),
        ('F4',   [1,0,1,1,1,1]),
        ('F#4',  [1,1,0,1,1,1]),
        ('G4',   [0,0,0,1,1,1]),
        ('G#4',  [1,1,1,0,1,1]),
        ('A4',   [0,0,0,0,1,1]),
        ('Bb4',  [0,0,1,1,0,1]),
        ('B4',   [0,0,0,0,0,1]),            
        ('C5',   [0,0,0,1,1,0]),                        
        ('C#5',  [0,0,0,0,0,0]), #Note: 0,0,1,0,0,0 lets you put a finger down           

        ('D5',   [1,1,1,1,1,0]),
        ('D5',   [1,1,1,1,1,1]),

        ('E5',   [0,1,1,1,1,1]),
        ('F5',   [1,0,1,1,1,1]),
        ('F#5',  [0,1,0,1,1,1]),
        ('G5',   [0,0,0,1,1,1]),
        #('G#5',  [0,0,1,0,1,1]),
        ('A5',   [0,0,0,0,1,1]),
       ('Bb5',  [1,1,1,0,1,1]), 
        ('B5',   [0,1,1,0,1,1]),
         
#        ('C6',   [1,1,1,0,1,0]),  # 1,0,1,0,1,0 may also be good 
        ('C6', [0,1,1,0,0,1]),
        #('C#6',  [1,1,1,0,0,0]),
        
        ('D6',   [1,1,1,1,1,1]), 
        #('E6',   [0,1,0,0,1,1]), 
    ],
        
    'folk' : [
        ('D4',   [1,1,1,1,1,1]),
        ('E4',   [0,1,1,1,1,1]),
        ('F#4',  [0,0,1,1,1,1]),
        ('G4',   [0,0,0,1,1,1]),
        ('A4',   [0,0,0,0,1,1]),
        ('B4',   [0,0,0,0,0,1]),
        ('C5',   [0,0,0,1,1,0]),
        ('C#5',  [0,0,0,0,0,0]),
        ('D5',   [1,1,1,1,1,0]),
        ('E5',   [0,1,1,1,1,1]),
        ('F#5',  [0,0,1,1,1,1]),
        ('G5',   [0,0,0,1,1,1]),
        ('A5',   [0,0,0,0,1,1]),
        ('B5',   [0,0,0,0,0,1]),
        #('C#6',  [1,1,1,0,0,0]),
        ('C#6',  [0,0,0,0,0,0]),
        ('D6',   [1,1,1,1,1,0]),

      #  ('D4*3',   [1,1,1,1,1,1]),
      #  ('E4*3',   [0,1,1,1,1,1]),
      #  ('F#4*3',  [0,0,1,1,1,1]),
      #  ('G4*3',   [0,0,0,1,1,1]),
      #  ('A4*3',   [0,0,0,0,1,1]),
        #('B4*3',   [0,0,0,0,0,1]),
        #('C#5*3',  [0,0,0,0,0,0]),

        #('E6',   [0,1,1,1,1,1]),
        #('F6',   [1,0,1,1,1,1]),
        #('G6',   [1,0,0,1,1,1]),
        #('A6',   [0,1,1,1,1,0]), #?        
    ],

    'minor' : [
        ('D4',   [1,1,1,1,1,1]),
        ('E4',   [0,1,1,1,1,1]),
        ('F4',   [0,0,1,1,1,1]),
        ('G4',   [0,0,0,1,1,1]),
        ('A4',   [0,0,0,0,1,1]),
        ('Bb4',  [0,0,0,0,0,1]),
        ('C5',   [0,0,0,0,0,0]),
        ('D5',   [1,1,1,1,1,1]),
        ('E5',   [0,1,1,1,1,1]),
        ('F5',   [0,0,1,1,1,1]),
        ('G5',   [0,0,0,1,1,1]),
        ('A5',   [0,0,0,0,1,1]),
        ('Bb5',  [0,0,0,0,0,1]),
        ('C6',   [0,0,0,0,0,0]),
        ('D6',   [1,1,1,1,1,1]),
    ],
    
    'dorian' : [
        ('D4',   [1,1,1,1,1,1]),
        ('E4',   [0,1,1,1,1,1]),
        ('F4',   [0,0,1,1,1,1]),
        ('G4',   [0,0,0,1,1,1]),
        ('A4',   [0,0,0,0,1,1]),
        ('Bb4',  [0,0,1,1,0,1]),
        ('B4',   [0,0,0,0,0,1]),
        ('C5',   [0,0,0,0,0,0]),
        ('D5',   [1,1,1,1,1,1]),
        ('E5',   [0,1,1,1,1,1]),
        ('F5',   [0,0,1,1,1,1]),
        ('G5',   [0,0,0,1,1,1]),
        ('A5',   [0,0,0,0,1,1]),
        ('Bb5',  [0,0,0,1,0,1]),
        ('B5',   [0,0,0,0,0,1]),
        ('C6',   [0,0,0,0,0,0]),
        ('D6',   [1,1,1,1,1,1]),
    ],
}

@config.Float_flag('embextra', 
    'Constant controlling extra effective height of the embouchure hole due to lips, etc. '
    'Small adjustments of this value will change the angle at which the flute needs to be blown '
    'in order to be in tune.')
class Flute_designer(design.Instrument_designer):
    closed_top = True
    
    fingering_system = 'pflute'
    
    @property
    def fingerings(self):
        return [
           (note, fingering+[0])
           for note, fingering in fingerings[self.fingering_system]
           ]
    
    # 2.5/8 = 0.3    ~ 20 cents flat
    # 5/8 = 0.6      ~ too high
    # 0.45 (sop_flute_5)   ~ a tiny bit low
    # 0.53 (sop_flute_8)   ~ a tiny biy low?, needed to push cork in 1.5mm
    #                        + perfect, printed plastic sop flute
    # 0.56                 ~ definitely high, printed plastic sop flute
    
    embextra = 0.53

    def patch_instrument(self, inst):
        inst = copy.copy(inst)        
        inst.hole_lengths[-1] += inst.hole_diameters[-1] * self.embextra
        return inst
        
#    
#    @property
#    def hole_extra_height_by_diameter(self):
#        return [ 0.0 ] * 6 + [ self.embextra ]
    
    #hole_extra_height_by_diameter = [ 0.0 ] * 6 + [ 0.53 ]
    
    initial_length = design.wavelength('D4') * 0.5
    
    initial_hole_fractions = [ 0.175 + 0.5*i/6 for i in range(6) ] + [ 0.97 ]

    
#    min_hole_diameters = design.sqrt_scaler([ 6.5 ] * 6  + [ 12.2 ])
#    max_hole_diameters = design.sqrt_scaler([ 11.4 ] * 6 + [ 13.9 ])
#    max_hole_diameters = design.sqrt_scaler([ 11.4 ] * 6 + [ 10.5 ])

    min_hole_diameters = design.power_scaler(1/3., [ 3.0 ] * 6  + [ 11.3 ])
    max_hole_diameters = design.power_scaler(1/3., [ 11.4 ] * 6 + [ 11.4 ])
    
    divisions = [
        [ (5, 0.0) ],
        [ (2, 0.0), (5, 0.0) ],
        [ (-1, 0.125), (2, 0.0), (5, 0.0), ],
        [ (-1, 0.125), (2,0.0), (5,0.0), (5,0.45) ],
        ]


@config.Float_flag('inner_taper', 'Amount of tapering of bore. Smaller = more tapered.')
@config.Float_flag('outer_taper', 'Amount of tapering of exterior. Smaller = more tapered.')
class Tapered_flute(Flute_designer):
    inner_taper = 0.75
    outer_taper = 0.85
    
    #inner_diameters = design.sqrt_scaler([ 14.0, 14.0, 18.4, 21.0, 18.4, 18.4 ])
    @property
    def inner_diameters(self):
        scale = self.scale ** (1./2)
        return [
            18.4 * self.inner_taper * scale,
            18.4 * self.inner_taper * scale,
            18.4 * (0.5+self.inner_taper*0.5) * scale,
            18.4 * scale,
            21.0 * scale,
            21.0 * scale,
            18.4 * scale,
            18.4 * scale,
            ]

    #initial_inner_fractions = [ 0.25, 0.75 ]
    #min_inner_fraction_sep = [ 0.0, 0.0, 0.0 ]
    
    initial_inner_fractions = [ 0.25,  0.3,  0.7, 0.8,0.81, 0.9 ]
    min_inner_fraction_sep = [ 0.01, 0.1,0.1, 0.01, 0.01, 0.01, 0.01 ]

    #outer_diameters = design.sqrt_scaler([ 22.1, 32.0, 26.1 ])
    @property
    def outer_diameters(self):
        scale = self.scale ** (1./2)
        return [
            29.0 * self.outer_taper * scale,
            29.0 * self.outer_taper * scale,
            29.0 * scale,
            29.0 * scale,
            #30.0 * scale,
            #30.0 * scale,
            #32.0 * scale,
            #29.0 * scale,
            ]

    initial_outer_fractions = [ 0.01, 0.666 ]
    min_outer_fraction_sep = [ 0.0, 0.5, 0.0 ] #Looks and feels nicer

#
#class Straight_flute(Flute_designer):
#    inner_diameters = design.sqrt_scaler([ 18.4, 18.4, 21.0, 18.4, 17.0 ])
#    initial_inner_fractions = [ 0.7, 0.8, 0.9 ]    
#    min_inner_fraction_sep = [ 0.5, 0.03, 0.0, 0.0 ]
#    # Note constraint of bulge to upper half of tube.
#    # There seems to be an alternate solution for the folk flute
#    #   where it's stretched out over 3/4 of the flute's length.
#
#    outer_diameters = design.sqrt_scaler([ 28.0, 28.0 ])
#
#    #initial_outer_fractions = [ 0.666 ]
#    #min_outer_fraction_sep = [ 0.666, 0.0 ] #Looks and feels nicer
#

@config.help(
    'Design a flute with a recorder-like fingering system.'
    )
class Design_pflute(Tapered_flute):
    fingering_system = 'pflute'
    balance = [ 0.1, None, None, 0.05 ]    
    #hole_angles = [ -30.0, -30.0, 30.0, -30.0, 30.0, -30.0, 0.0 ]
    #hole_angles = [ 30.0, -30.0, 30.0, 0.0, 0.0, 0.0, 0.0 ]
    hole_angles = [ 30.0, -30.0, 30.0, 0.0, 0.0, 0.0, 0.0 ]

    max_hole_spacing = design.scaler([ 45.0, 45.0, None, 45.0, 45.0, None ])


#@config.help("""\
#Design a flute with a tapered bore and recorder-like fingering system.
#""")
#class Design_tapered_pflute(Tapered_flute, Pflute): pass
#
#@config.help("""\
#Design a flute with a straight bore and recorder-like fingering system.
#""")
#class Design_straight_pflute(Straight_flute, Pflute): pass

@config.help(
    'Design a flute with a pennywhistle-like fingering system.'
    )
class Design_folk_flute(Tapered_flute):
    fingering_system = 'folk'
    balance = [ 0.01, None, None, 0.01 ]    
    hole_angles = [ -30.0, 30.0, 30.0,  -30.0, 0.0, 30.0, 0.0 ]
    #min_hole_diameters = design.sqrt_scaler([ 7.5 ] * 6  + [ 12.2 ])
    #max_hole_diameters = design.sqrt_scaler([ 11.4 ] * 6 + [ 13.9 ])
    
    max_hole_spacing = design.scaler([ 35.0, 35.0, None, 35.0, 35.0, None ])

#@config.help("""\
#Design a flute with a tapered bore and pennywhistle-like fingering system.
#""")
#class Design_tapered_folk_flute(Tapered_flute, Folk_flute): pass
#
#@config.help("""\
#Design a flute with a straight bore and pennywhistle-like fingering system.
#""")
#class Design_straight_folk_flute(Straight_flute, Folk_flute): pass
#

#
#class With_tuning_holes(Design_pflute):
#    #@property
#    #def n_holes(self):
#    #    x = super(With_tuning_holes,self).n_holes
#    #    print x
#    #    return 1+x
#    
#    tuning_holes = 2
#    
#    @property
#    def min_hole_diameters(self):
#        x = super(With_tuning_holes,self).min_hole_diameters
#        return [ 0.1 ]*self.tuning_holes+x
#    @property
#    def max_hole_diameters(self):
#        x = super(With_tuning_holes,self).max_hole_diameters
#        d = self.inner_diameters[0]
#        return [d*0.5]*self.tuning_holes+x
#    @property
#    def balance(self):
#        x = super(With_tuning_holes,self).balance
#        return [None]*self.tuning_holes+x
#    @property
#    def max_hole_spacing(self):
#        x = super(With_tuning_holes,self).max_hole_spacing
#        return [None]*self.tuning_holes+x
#    @property
#    def initial_hole_fractions(self):
#        x = super(With_tuning_holes,self).initial_hole_fractions
#        return [ (i+1.0)/(self.tuning_holes+1.0) for i in xrange(self.tuning_holes) ]+x
#    @property
#    def hole_angles(self):
#        x = super(With_tuning_holes,self).hole_angles
#        return [0.0]*self.tuning_holes+x
#    @property
#    def fingerings(self):
#        x = super(With_tuning_holes,self).fingerings
#        return [ (a,[0]*self.tuning_holes+b) for a,b in x ]
#










#!/usr/bin/env python

import os

#from distutils.core import setup
from setuptools import setup

#Read version
with open(os.path.normpath(os.path.join(__file__,'..','demakein','__init__.py')),'rU') as f:
    exec f.readline()

setup(name='demakein',
      version=VERSION,
      description='Design woodwind instruments and make them with a 3D printer or CNC mill.',
      
      packages = [
          'demakein', 
          'demakein.raphs_curves',
      ],
      
      entry_points = {
          'console_scripts' : [
              'demakein = demakein:main',
          ],
      },

      classifiers = [
          'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
      ],
      
      install_requires = [ 
          'nesoni', 
          'cffi' ,
      ],
      
      url = 'http://www.logarithmic.net/pfh/design',
      author = 'Paul Harrison',
      author_email = 'pfh@logarithmic.net',
)




import bisect, math
from raphs_curves import cornu


class Profile:
    def __init__(self, pos, low, high=None):
        if high is None: high = low
        self.pos = pos
        self.low = low
        self.high = high
    

    def __call__(self, position, high=False):
        if position < self.pos[0]:
            return self.low[0]
        if position > self.pos[-1]:
            return self.high[-1]
        i = bisect.bisect_left(self.pos, position)
        if self.pos[i] == position:
            if high:
                 return self.high[i]
            else:
                 return self.low[i]

        t = float(position - self.pos[i-1])/(self.pos[i]-self.pos[i-1])
        return (1.0-t)*self.high[i-1] + t*self.low[i]


    def __repr__(self):
        return 'Profile' + repr((self.pos,self.low,self.high))


    def start(self):
        return self.pos[0]
    

    def end(self):
        return self.pos[-1]


    def maximum(self):
        return max(max(self.low),max(self.high))


    def morph(self, other, operator):
        """ Fairly dumb way to combine profiles. 
            Won't work perfectly for min, max. """
        if not isinstance(other, Profile):
            other = Profile([0.0],[other],[other])
        pos = sorted(set(self.pos + other.pos))
        low = [ operator(self(p,False),other(p,False)) for p in pos ]
        high = [ operator(self(p,True),other(p,True)) for p in pos ]
        return Profile(pos,low,high)

    def max_with(self, other):
        return self.morph(other, lambda a,b: max(a,b))

    def min_with(self, other):
        return self.morph(other, lambda a,b: min(a,b))

    def __add__(self, other):
        return self.morph(other, lambda a,b: a+b)

    def __sub__(self, other):
        return self.morph(other, lambda a,b: a-b)

    def clipped(self, start, end):
        """ Clip or extend a profile """
        
        pos = [ start ]
        low = [ self(start, True) ]
        high = [ self(start, True) ]
        
        for i, p in enumerate(self.pos):
            if p <= start or p >= end: continue
            pos.append(p)
            low.append(self.low[i])
            high.append(self.high[i])
        
        pos.append(end)
        low.append(self(end, False))
        high.append(self(end, False))
        return Profile(pos,low,high)

    def reversed(self):
        """ Reverse profile. Positions are negated. """
        new_pos = [ -item for item in self.pos[::-1] ]
        return Profile(new_pos, self.high[::-1], self.low[::-1])

    def moved(self, offset):
        new_pos = [ item+offset for item in self.pos ]
        return Profile(new_pos, self.low, self.high)

    def as_stepped(self, max_step):
        pos = [ ]
        
        low = [ ]
        high = [ ]

        for i in range(len(self.pos)-1):
            pos.append(self.pos[i])
        
            ax = self.pos[i]
            ay = self.high[i]
            bx = self.pos[i+1]
            by = self.low[i+1]
            n = int( float(abs(by - ay)) / max_step )+1
            if not n: continue            
            pos.extend( (bx-ax)*float(j)/n+ax for j in range(1,n) )

        pos.append(self.pos[-1])
        
        diams = [ self(0.5*(pos[i]+pos[i+1])) for i in range(len(pos)-1) ]
        low = [ diams[0] ] + diams
        high = diams + [ diams[-1] ]
        
        #for i in range(len(pos)-1):
        #    assert high[i] == low[i+1], repr((high[i],low[i+1]))
        
        return Profile(pos,low,high)


def length(x,y):
    return math.sqrt(x*x+y*y)

def cornu_yx(t,mirror):
    # Reparamaterize for constant absolute rate of turning
    t = math.sqrt(abs(t)) * (1 if t > 0 else -1)
    y,x = cornu.eval_cornu(t)
    if mirror: y = -y
    return y,x

def solve(a1,a2):        
    pi = math.pi
    two_pi = pi*2
    
    def score(t1,t2,mirror):
        if abs(t1-t2) < 1e-6 or max(abs(t1),abs(t2)) > pi*10.0: return 1e30
    
        y1,x1 = cornu_yx(t1, mirror)
        y2,x2 = cornu_yx(t2, mirror)
        chord_a = math.atan2(y2-y1,x2-x1)
        chord_l = length(y2-y1, x2-x1)
        this_a1 = abs(t1) #t1*t1
        this_a2 = abs(t2) #t2*t2
        if mirror:
            this_a1 = -this_a1
            this_a2 = -this_a2
        if t1 > t2:
            this_a1 += pi
            this_a2 += pi
        ea1 = (this_a1-chord_a-a1+pi)%two_pi - pi
        ea2 = (this_a2-chord_a-a2+pi)%two_pi - pi
        return ea1*ea1+ea2*ea2
    
    s = None
    n = 2
    for new_mirror in [False,True]:
        for i in range(-n,n+1):
            for j in range(-n,n+1):    
                new_t1 = i*pi/n
                new_t2 = j*pi/n
                new_s = score(new_t1,new_t2,new_mirror)
                if s is None or new_s < s:
                    t1 = new_t1
                    t2 = new_t2
                    mirror = new_mirror
                    s = new_s
       
    step = pi / n * 0.5
    while step >= 1e-4:
        for new_t1,new_t2 in [(t1+step,t2+step), (t1-step,t2-step), (t1-step,t2+step), (t1+step,t2-step)]:
            new_s = score(new_t1,new_t2,mirror)
            if new_s < s:
                s = new_s
                t1 = new_t1
                t2 = new_t2
                break
        else:
            step *= 0.5

    return t1, t2, mirror

def curved_profile(pos, low, high, low_angle, high_angle, quality=512):
    n = len(pos)

    a = [ ]
    for i in range(n-1):
        x1 = pos[i]
        y1 = high[i] * 0.5
        x2 = pos[i+1]
        y2 = low[i+1] * 0.5
        a.append( (math.atan2(y2-y1,x2-x1)+math.pi)%(math.pi*2)-math.pi )    
    def interpret(i,value):
        if value == None:
            return None
        if value == 'mean':
            return (a[i-1]+a[i])*0.5
        if value == 'up':
            return a[i]
        if value == 'down':
            return a[i-1]
        return value*math.pi/180
    low_angle = [ interpret(i,value) for i,value in enumerate(low_angle) ]
    high_angle = [ interpret(i,value) for i,value in enumerate(high_angle) ]
    
    ppos = [ ]
    plow = [ ]
    phigh = [ ]
    
    for i in range(n-1):
        ppos.append(pos[i])
        plow.append(low[i])
        phigh.append(high[i])
        
        x1 = pos[i]
        y1 = high[i] * 0.5
        x2 = pos[i+1]
        y2 = low[i+1] * 0.5
        l = length(x2-x1,y2-y1)
        a = math.atan2(y2-y1,x2-x1)
        
        if high_angle[i] is not None:
            a1 = high_angle[i] - a
        else:
            a1 = 0.0
            
        if low_angle[i+1] is not None:
            a2 = low_angle[i+1] - a
        else:
            a2 = 0.0
        
        if abs(a1-a2) < math.pi*2/quality: continue
        
        #t1 = th1
        #t2 = th2
        
        #k0,k1 = clothoid.solve_clothoid(th1/math.pi,th2/math.pi)
        #print(th1, th2, k0, k1)
        
        #if abs(k1) < 1e-6: continue
        
        #t1 = k0-k1*0.5
        #t2 = k0+k1*0.5
        
        t1, t2, mirror = solve(a1,a2)
        
        cy1,cx1 = cornu_yx(t1,mirror)
        cy2,cx2 = cornu_yx(t2,mirror)
        cl = length(cx2-cx1,cy2-cy1)
        if abs(cl) < 1e-10: continue
        
        ca = math.atan2(cy2-cy1,cx2-cx1)
        
        steps = int( abs(t2-t1) / (math.pi*2) * quality )
        for i in range(1,steps):
            t = t1+i*(t2-t1)/steps
            yy,xx = cornu_yx(t,mirror)
            aa = math.atan2(yy-cy1,xx-cx1)
            ll = length(yy-cy1,xx-cx1)            
            x = math.cos(aa-ca+a) * ll/cl*l +x1
            y = math.sin(aa-ca+a) * ll/cl*l +y1
            ppos.append(x)
            plow.append(y*2)
            phigh.append(y*2)
    
    ppos.append(pos[-1])
    plow.append(low[-1])
    phigh.append(high[-1])
    return Profile(ppos, plow, phigh)



def make_profile(spec):
    pos = [ ]
    low = [ ]
    high = [ ]
    for item in spec:
        if len(item) == 2:
            this_pos, this_low = item
            this_high = this_low
        else:
            this_pos, this_low, this_high = item
        pos.append(this_pos)
        low.append(this_low)
        high.append(this_high)
    return Profile(pos, low, high)

if __name__ == '__main__':
    from raphs_curves import cornu
    
    for i in range(20+1):
        t = i / 10.0 - 0.5
        
        y1,x1 = cornu.eval_cornu(t)
        y2,x2 = cornu.eval_cornu(t+1e-3)
        print( t, (math.atan2(y2-y1,x2-x1)+math.pi) % (2*math.pi) -math.pi )   #Angle is t^2





import math, traceback, os, random, sys, io, collections

from . import profile, design, pack, geom

from nesoni import config

#=====================
#from blender import *
#=====================
from cgal import *
#=====================


# We use mm internally

#SCALE = 0.001   # mm -> m
#SCALE = 1.0   # mm -> mm

QUALITY = 128
def draft_mode():
    global QUALITY
    QUALITY = 16

NAME_COUNT = 0
def make_name(value=None):
    global NAME_COUNT
    if value: return value
    NAME_COUNT += 1
    return 'obj%d' % NAME_COUNT

CLEAN_MLX = os.path.join(os.path.split(__file__)[0], 'clean.mlx')


def noise():
    return (random.random()-0.5)*1e-4

Limits = collections.namedtuple('Limits',
   'xmin xmax ymin ymax zmin zmax'
)

Limits_2 = collections.namedtuple('Limits_2',
   'xmin xmax ymin ymax'
)

class Loop(list):
    @property
    def circumpherence(self):
        total = 0.0
        last = self[-1]
        for point in self:
            dx = last[0]-point[0]
            dy = last[1]-point[1]
            total += math.sqrt(dx*dx+dy*dy)
            last = point
        return total

    @property
    def area(self):
        total = 0.0
        last = self[-1]
        for point in self:
            total += (
                last[0]*point[1]
              - last[1]*point[0]
            )
            last = point
        return 0.5*total
    
    @property
    def centroid(self):
        div = 6.0 * self.area
        sum_x = 0.0
        sum_y = 0.0
        div = 0.0
        last = self[-1]
        for point in self:
            val = last[0]*point[1]-point[0]*last[1]
            div += val
            sum_x += (last[0]+point[0])*val
            sum_y += (last[1]+point[1])*val
            last = point
        div *= 3.0
        
        if not div: # Probably a point, possibly a line, do something vaguely sensible
           div = len(self)
           sum_x = sum(point[0] for point in self)
           sum_y = sum(point[1] for point in self)
        
        return (sum_x/div, sum_y/div)

    def extent(self):
        xs = [ item[0] for item in self ]
        ys = [ item[1] for item in self ]
        return Limits_2(
            min(xs),max(xs),min(ys),max(ys)
            )
    
    def scale(self, factor):
        return Loop( (x*factor, y*factor) for x,y in self )

    def scale2(self, factor_x, factor_y):
        return Loop( (x*factor_x, y*factor_y) for x,y in self )
    
    def with_area(self, area):
        return self.scale(math.sqrt( area/self.area ))
    
    def with_effective_diameter(self, diameter):
        return self.with_area(math.pi*0.25*diameter*diameter)
    
    def with_circumpherence(self, circumpherence):
        return self.scale(circumpherence / self.circumpherence)

    def offset(self, dx,dy):
        return Loop( (x+dx,y+dy) for x,y in self )

    def flip_x(self):
        return Loop( (-x,y) for x,y in self[::-1] )
        
    def flip_y(self):
        return Loop( (x,-y) for x,y in self[::-1] )

    def mask(self, res):
        import mask
        lines = [ (self[i][0]*res,self[i][1]*res,
                   self[(i+1)%len(self)][0]*res,self[(i+1)%len(self)][1]*res)
                  for i in range(len(self)) ]
        return mask.make_mask(lines)

    #def mean(self):
    #    return (float(sum( x for x,y in self )) / len(self),
    #            float(sum( y for x,y in self )) / len(self))

    def shape_2(self):
        return create_polygon_2(self)
    
    def shape_3(self):
        return self.shape_2().to_3()


def circle(diameter=1.0, n=None):
    if n is None: n = QUALITY
    radius = diameter * 0.5
    result = [ ]
    for i in range(n):
        a = (i+0.5)*math.pi*2.0/n
        result.append( (math.cos(a)*radius, math.sin(a)*radius) )
    return Loop(result)


def chorded_circle(amount=0.5):
    """ semi-circle and the like """
    result = [ ]
    a1 = math.pi*(0.5+amount)
    a2 = math.pi*(2.5-amount)
    for i in range(QUALITY):
        a = a1+(i+0.5)*(a2-a1)/QUALITY
        result.append( (math.cos(a), math.sin(a)) )
    return Loop(result)


def square(size):
    return Loop([
        (size,size),
        (-size,size),
        (-size,-size),
        (size,-size),
    ])

def squared_circle(xpad,ypad, diameter=1.0):
    """ Squared circle with same area as circle of specified diameter """
    result = [ ]
    for i in range(QUALITY):
        a = (i+0.5)*math.pi*2.0/QUALITY
        x = math.cos(a)
        if x < 0: x -= xpad*0.5
        else: x += xpad*0.5
        y = math.sin(a)
        if y < 0: y -= ypad*0.5
        else: y += ypad*0.5
        result.append( (x,y) )
    
    area = math.pi + xpad*ypad + xpad*2 + ypad*2
    want = math.pi * (diameter*0.5)**2
    scale = math.sqrt( want/area )
    result = [ (x*scale,y*scale) for x,y in result ]
    
    return Loop(result)

def rectangle(x0,x1,y0,y1):
    return Loop([
        (x0,y0),
        (x1,y0),
        (x1,y1),
        (x0,y1)
        ])

def rounded_rectangle(x0,x1,y0,y1, diameter):    
    radius = min(diameter,x1-x0,y1-y0)*0.5 
    result = [ ]
    for i in range(QUALITY):
        a = (i+0.5)*math.pi*2.0/QUALITY
        x = math.cos(a)*radius
        if x < 0: x += x0+radius
        else: x += x1-radius
        y = math.sin(a)*radius
        if y < 0: y += y0+radius
        else: y += y1-radius
        result.append( (x,y) )
    return Loop(result)

def halfrounded_rectangle(x0,x1,y0,y1):
    radius = x1-x0
    result = [ ]
    for i in range(QUALITY):
        a = ((i+0.5)/QUALITY-0.5)*math.pi
        x = math.cos(a)*radius + x0
        y = math.sin(a)*radius
        if y < 0: y += y0+radius
        else: y += y1-radius
        result.append( (x,y) )
    result.extend([ (x0,y1), (x0,y0) ])
    return Loop(result)

def lens(amount, circumpherence=math.pi):
    result = [ ]
    turn = math.asin(amount)
    turn2 = math.pi-turn*2
    shift = math.sin(turn)
    for i in range(QUALITY // 2):
        a = (i+0.5)/QUALITY*2 * turn2 + turn
        result.append((
            math.cos(a),
            math.sin(a) - shift
        ))
    result = Loop(
        result +
        [ (-x,-y) for x,y in result ]
    )
    return result.with_circumpherence(circumpherence)

def lens2(amount, circumpherence=math.pi):
    result = [ ]
    turn = math.pi*0.5*amount
    turn2 = math.pi-turn*2
    shift = math.sin(turn)
    for i in range(QUALITY // 2):
        a = (i+0.5)/QUALITY*2 * turn2 + turn
        result.append((
            math.cos(a),
            math.sin(a) - shift
        ))
    result = Loop(
        result +
        [ (-x,-y) for x,y in result ]
    )
    return result.with_circumpherence(circumpherence)

#def loop_merge(loop0, loop1):
#
#def extrusion(zs, shapes, name=None):
#    centroids = [ item.centroid for item in shapes ]
#    
#    faces = [ ]
#    verts = [ ]
#    vert_index = { }
#    def vert(point):
#        if point not in vert_index:
#            vert_index[point] = len(verts)
#            verts.append(point)
#        return vert_index[point]
#    
#    shape_vert = [ [ vert(item+(z,)) for item in shape ] for shape,z in zip(shapes,zs) ]
#    
#    end0 = vert(shapes[0].centroid+(zs[0],))
#    end1 = vert(shapes[-1].centroid+(zs[-1],))
#    
#    for i in xrange(len(zs)):
#        ...
#    
#    for i in xrange(len(shapes[0])):
#        j = (i+1)%len(shapes[0])
#        faces.append( (end0,shape_vert[0][i],shape_vert[0][j]) )
#    for i in xrange(len(shapes[-1])):
#        j = (i+1)%len(shapes[-1])
#        faces.append( (end1,shape_vert[-1][i],shape_vert[-1][j]) )
#    
#    return create(verts, faces, name)

def extrusion(zs, shapes, name=None):
    nz = len(zs)
    nshape = len(shapes[0])
    
    verts = [ ]
    for i,z in enumerate(zs):
        for x,y in shapes[i]:
            verts.append( (x,y,z) )
    
    end0 = len(verts)
    verts.append( shapes[0].centroid + (zs[0],) )
    end1 = len(verts)
    verts.append( shapes[-1].centroid + (zs[-1],) )
    
    
    faces = [ ]
    for i in range(nz-1):
        for j in range(nshape):
            #faces.append( ( 
            #    (i+1)*nshape+j, i*nshape+j,
            #    i*nshape+(j+1)%nshape, (i+1)*nshape+(j+1)%nshape 
            #) )
            faces.append( ( 
                (i+1)*nshape+j, i*nshape+j,
                i*nshape+(j+1)%nshape, 
            ) )
            faces.append( ( 
                (i+1)*nshape+j,
                i*nshape+(j+1)%nshape, (i+1)*nshape+(j+1)%nshape 
            ) )

    for i in range(nshape):
        i1 = (i+1)%nshape
        faces.append( (i1,i,end0) )
        faces.append( (i+nshape*(nz-1),i1+nshape*(nz-1),end1) )

    return create(verts, faces, name)

def block(x1,x2,y1,y2,z1,z2, name=None, ramp=0.0):
    verts = [ ]
    for x in [x1,x2]:
        for y in [y1,y2]:
            verts.append((x,y,z1))
    for x in [x1-ramp,x2+ramp]:
        for y in [y1-ramp,y2+ramp]:
            verts.append((x,y,z2))
        
    faces = [ ]

    def quad(a,b,c,d):
        faces.append((a,b,c))
        faces.append((a,c,d))

    for a,b,c in [ (1,2,4),(4,1,2),(2,4,1) ]:
        quad(0,a,a+b,b)
        quad(c+b,c+a+b,c+a,c+0)

    return create(verts, faces, name)


def extrude_profile(*profiles, **kwargs):
    for arg in kwargs: assert arg in ('cross_section','name')
    cross_section = kwargs.get('cross_section',circle)
    name = kwargs.get('name',None)
    
    zs = [ ]
    shapes = [ ]
    
    pos = set()
    for item in profiles:
        pos.update(item.pos)
    pos = sorted(pos)
    
    for i,z in enumerate(pos):
        lows = [ item(z) for item in profiles ]
        highs = [ item(z,True) for item in profiles ]
    
        if i:
            zs.append(z)
            shapes.append(cross_section(*lows))
        if not i or (i < len(pos)-1 and lows != highs):
            zs.append(z)            
            shapes.append(cross_section(*highs))
    
    return extrusion(zs, shapes, name=name)


def prism(height, diameter, cross_section=circle, name=None):
    span = profile.Profile([0.0, height], [diameter, diameter])
    return extrude_profile(span, cross_section=cross_section, name=name)


def make_segment(instrument, top, low, high, radius, pad=0.0, clip_half=True):
    if not clip_half:
       y1,y2 = -radius,radius
    elif top:
       y1,y2 = 0,radius
    else:
       y1,y2 = -radius,0
    clip = block(-radius,radius, y1,y2, low-pad,high+pad)
    segment = instrument.copy()
    segment.clip(clip)
    assert abs(segment.size()[2] - (high-low+pad*2)) < 1e-3, 'Need more padding for construction'
    segment.move(0,0,-low)
    if top:
       segment.rotate(0,0,1, 180)

    segment.rotate(1,0,0,-90)
    segment.rotate(0,0,1,90)
    segment.move(high-low,0,0)
    return segment


def make_segments(instrument, length, radius, top_fractions, bottom_fractions, pad=0.0, clip_half=True):
    parts = [ ]
    lengths = [ ]
    z = [ item*length for item in top_fractions ]
    for i in range(len(top_fractions)-1):
        lengths.append(z[i+1]-z[i])    
        parts.append(make_segment(instrument, True, 
                     z[i],z[i+1], 
                     radius, pad, clip_half))

    z = [ item*length for item in bottom_fractions ]
    for i in range(len(bottom_fractions)-1):    
        lengths.append(z[i+1]-z[i])    
        parts.append(make_segment(instrument, False, 
                     z[i],z[i+1], 
                     radius, pad, clip_half))

    return parts, lengths

def make_formwork(outer, bore, length, top_fractions, bottom_fractions, 
                  bit_diameter, peg_diameter, depth, width):
    """ Object must be large enough to include end padding """
    xsize,ysize,zsize = outer.size()
    radius = max(xsize,ysize)
    
    pad = bit_diameter * 1.25
    ramp = depth / 10.0
    
    peg_margin = max(peg_diameter, ramp)
    
    eps = 1e-4
    
    outers, lengths = make_segments(outer, length, radius, top_fractions, bottom_fractions, 0)
    bores, _ = make_segments(bore, length, radius, top_fractions, bottom_fractions, bit_diameter, False)

    n = len(outers)
    order = sorted(range(n),key=lambda i: -lengths[i])
    lengths = [ lengths[i] for i in order ]
    outers = [ outers[i] for i in order ]
    bores = [ bores[i] for i in order ]

    places = pack.pack([ item.size() for item in outers], pad, width)
    places = [ (x,y+peg_margin) for x,y in places ]

    xsize = 0
    ysize = 0
    for i in range(n):
        xmin,xmax,ymin,ymax,zmin,zmax = outers[i].extent()
        outers[i].move(places[i][0], places[i][1]-ymin, 0)
        bores[i] .move(places[i][0], places[i][1]-ymin, -eps)
        
        xsize = max(xsize,xmax-xmin+places[i][0]+pad)
        ysize = max(ysize,ymax-ymin+places[i][1]+pad)
        
    
    #y = peg_margin + pad
    #xsize = 0
    #for i in range(n):
    #    xmin,xmax,ymin,ymax,zmin,zmax = outers[i].extent()
    #    assert zmax <= depth, 'Too big: %.1f' % zmax
    #    assert abs(zmin) < 1e-3, 'Wut?'
    #    
    #    outers[i].move(pad,-ymin+y,0)
    #    bores[i].move(pad,-ymin+y,0)
    #    y += ymax-ymin+pad
    #    xsize = max(xsize,lengths[i]+pad*2)
    #
    #ysize = y
    
    # Clip tops of bollards... (0.5 max height, pretty small)
    #for i in range(n):
    #    xmin,xmax,ymin,ymax,zmin,zmax = outers[i].extent()
    #    left_height = outers[i].max_z_near_x(xmin)
    #    right_height = outers[i].max_z_near_x(xmax)
    #    
    #    outers[i].remove(block(
    #        xmin-eps,pad,
    #        ymin-eps,ymax+eps,
    #        min(0.5,left_height*0.5), depth
    #    ))
    #    
    #    outers[i].remove(block(
    #        pad+lengths[i],xmax+eps,
    #        ymin-eps,ymax+eps,
    #        min(0.5,right_height*0.5), depth
    #    ))
        
    
    lower = block(-ramp,xsize+ramp,0,ysize+ramp,0,depth+1)
    upper = block(-ramp,xsize+ramp,0,ysize+ramp,eps,depth)
    
    print('Necessary origin: x +', ramp, ' y + ', peg_margin-peg_diameter)
    
    for item in bores:
        lower.remove(item)
    
    hole1 = prism(peg_diameter*1.5, peg_diameter)
    hole2 = hole1.copy()
    
    hole1.move(peg_diameter,peg_margin-peg_diameter*0.5,-eps)
    lower.remove(hole1)
    
    hole2.move(xsize-peg_diameter,peg_margin-peg_diameter*0.5,-eps)
    lower.remove(hole2)
    

    #upper.remove(block(
    #    -eps,xsize+eps, peg_diameter+eps,ysize+eps, 0,depth
    #))
    
    xmins = [ ]
    xmaxs = [ ]
    for i in range(n):
        xmin,xmax,ymin,ymax,zmin,zmax = outers[i].extent()
        xmins.append(xmin)
        xmaxs.append(xmax)
        
        upper.remove(block(
            xmin-pad-eps*(i+1),xmax+pad+eps*(i+1),
            ymin-pad+eps*(i+1),ymax+pad-eps*(i+1), # + (1 if i == n-1 else 0),
            #eps*(i+1),depth+eps,
            0,depth+eps,
            ramp = ramp
        ))

        #Purely to appease boolean ops library:
        outers[i].clip(block(
            xmin+eps*(i+2),xmax-eps*(i+2),
            ymin-pad*0.5,ymax+pad*0.5,
            -1,depth,
        ))
        
    for i in range(n):
        upper.add(outers[i])

    bol = int(xsize / 30.0)
    bol_width = 1.0
    bol_height = 1.5 
        # 0.5 was too low, middle piece tore loose in final pass of
        # contour finishing (x-scan of horizontal-like surfaces)
    for i in range(bol):
        pos = (i+0.5)*xsize/bol
        
        if any( pos-pad <= item <= pos+pad for item in xmins+xmaxs ):
            continue
        
        upper.add(block(
            pos-bol_width*0.5, pos+bol_width*0.5,
            eps,ysize+ramp-eps,
            -bit_diameter*0.6, bol_height,
        ))     
    
    #Flip bore formwork after construction
    lower.rotate(0,1,0, 180)
    
    return lower, upper




def frame_extrusion(frames, shapes):
    nframes = len(frames)
    nshape = len(shapes[0])
    
    verts = [ ]
    for i,frame in enumerate(frames):
        for x,y in shapes[i]:
            verts.append( frame.apply(geom.XYZ(x,y,0.0)) )
    
    end0 = len(verts)
    verts.append( frames[0].apply(geom.XYZ(*shapes[0].centroid + (0.0,))) )
    end1 = len(verts)
    verts.append( frames[-1].apply(geom.XYZ(*shapes[-1].centroid + (0.0,))) )
        
    faces = [ ]
    for i in range(nframes-1):
        for j in range(nshape):
            #faces.append( ( 
            #    (i+1)*nshape+j, i*nshape+j,
            #    i*nshape+(j+1)%nshape, (i+1)*nshape+(j+1)%nshape 
            #    ) )
            faces.append( ( 
                (i+1)*nshape+j, i*nshape+j,
                i*nshape+(j+1)%nshape, 
                ) )
            faces.append( ( 
                (i+1)*nshape+j,
                i*nshape+(j+1)%nshape, (i+1)*nshape+(j+1)%nshape 
                ) )

    for i in range(nshape):
        i1 = (i+1)%nshape
        faces.append( (i1,i,end0) )
        faces.append( (i+nshape*(nframes-1),i1+nshape*(nframes-1),end1) )

    return create(verts, faces)


def path_extrusion(path, cross_section, *profiles):
    poses = set()
    for item in profiles:
        poses.update(item.pos)
    poses = sorted(poses)
    
    i = 0
    while i < len(poses)-1:
        #print poses[i+1], poses[i]
        #ratio = (path.get_point(poses[i+1])-path.get_point(poses[i])).mag() / abs(poses[i+1]-poses[i])
        #print 'r',ratio
        #if abs(poses[i+1]-poses[i]) > 1.0 and ratio < 1.0:
        b = path.get_bentness(poses[i],poses[i+1])
        print 'b', b
        if b > 1.0/QUALITY:
            poses.insert(i+1, (poses[i]+poses[i+1])*0.5)
        else:
            i += 1
    
    print poses
    
    frames = [ ]
    shapes = [ ]
    
    for i,pos in enumerate(poses):
        lows = [ item(pos) for item in profiles ]
        highs = [ item(pos,True) for item in profiles ]
    
        if i:
            frames.append(path.get_frame(pos))
            shapes.append(cross_section(*lows))
        if not i or (i < len(poses)-1 and lows != highs):
            frames.append(path.get_frame(pos))            
            shapes.append(cross_section(*highs))
    
    return frame_extrusion(frames, shapes)













